@@ -1,9 +1,7 @@
 #!/usr/bin/env python
 
 import configparser
-import copy
 import datetime
-from functools import wraps
 import getpass
 import json
 import logging
@@ -16,32 +14,27 @@ import tempfile
 import textwrap
 import time
 import traceback
-
+from functools import wraps
 
 import click
+import yaml
 from click.exceptions import ClickException
 
 # pycreds has a find_executable that works in windows
 from dockerpycreds.utils import find_executable
-import wandb
-from wandb import Config
-from wandb import env, util
-from wandb import Error
-from wandb import wandb_agent
-from wandb import wandb_sdk
 
+import wandb
+
+# from wandb.old.core import wandb_dir
+import wandb.sdk.verify.verify as wandb_verify
+from wandb import Config, Error, env, util, wandb_agent, wandb_sdk
 from wandb.apis import InternalApi, PublicApi
 from wandb.errors import ExecutionError, LaunchError
 from wandb.integration.magic import magic_install
 from wandb.sdk.launch.launch_add import _launch_add
-from wandb.sdk.launch.utils import construct_launch_spec
+from wandb.sdk.launch.utils import check_logged_in, construct_launch_spec
 from wandb.sdk.lib.wburls import wburls
-
-# from wandb.old.core import wandb_dir
-import wandb.sdk.verify.verify as wandb_verify
-from wandb.sync import get_run_from_path, get_runs, SyncManager, TMPDIR
-import yaml
-
+from wandb.sync import TMPDIR, SyncManager, get_run_from_path, get_runs
 
 # Send cli logs to wandb/debug-cli.<username>.log by default and fallback to a temp dir.
 _wandb_dir = wandb.old.core.wandb_dir(env.get_dir())
@@ -65,7 +58,14 @@ logging.basicConfig(
 )
 logging.basicConfig(stream=sys.stdout, level=logging.INFO)
 logger = logging.getLogger("wandb")
-CONTEXT = dict(default_map={})
+
+# Click Contexts
+CONTEXT = {"default_map": {}}
+RUN_CONTEXT = {
+    "default_map": {},
+    "allow_extra_args": True,
+    "ignore_unknown_options": True,
+}
 
 
 def cli_unsupported(argument):
@@ -99,11 +99,7 @@ def display_error(func):
             exc_type, exc_value, exc_traceback = sys.exc_info()
             lines = traceback.format_exception(exc_type, exc_value, exc_traceback)
             logger.error("".join(lines))
-            wandb.termerror(
-                "Find detailed error logs at: {}".format(
-                    os.path.join(_wandb_dir, "debug-cli.log")
-                )
-            )
+            wandb.termerror(f"Find detailed error logs at: {_wandb_log_path}")
             click_exc = ClickWandbException(e)
             click_exc.orig_type = exc_type
             raise click_exc.with_traceback(sys.exc_info()[2])
@@ -303,7 +299,7 @@ def service(
 @click.pass_context
 @display_error
 def init(ctx, project, entity, reset, mode):
-    from wandb.old.core import _set_stage_dir, __stage_dir__, wandb_dir
+    from wandb.old.core import __stage_dir__, _set_stage_dir, wandb_dir
 
     if __stage_dir__ is None:
         _set_stage_dir("wandb")
@@ -667,14 +663,10 @@ def sync(
 @click.option("--settings", default=None, help="Set sweep settings", hidden=True)
 @click.option("--update", default=None, help="Update pending sweep")
 @click.option(
-    "--queue",
-    "-q",
-    is_flag=False,
-    flag_value="default",
-    default=None,
-    help="Name of launch run queue to push sweep runs into. If supplied without "
-    "an argument (`--queue`), defaults to classic sweep behavior. Else, if "
-    "name supplied, specified run queue must exist under the project and entity supplied.",
+    "--launch_config",
+    "-c",
+    metavar="FILE",
+    help="Path to JSON or YAML file which defines how to launch the sweep.",
 )
 @click.option(
     "--stop",
@@ -712,7 +704,7 @@ def sweep(
     program,
     settings,
     update,
-    queue,
+    launch_config,
     stop,
     cancel,
     pause,
@@ -871,41 +863,65 @@ def sweep(
     )
 
     _launch_scheduler_spec = None
-    if queue is not None:
-        wandb.termlog("Using launch 🚀 queue: %s" % queue)
+    if launch_config is not None:
+        launch_config = util.load_json_yaml_dict(launch_config)
+        if launch_config is None:
+            raise LaunchError(f"Invalid format for launch config at {launch_config}")
+        wandb.termlog(f"Using launch 🚀 with config: {launch_config}")
 
-        # Because the launch job spec below is the Scheduler, it
-        # will need to know the name of the sweep, which it wont
-        # know until it is created,so we use this placeholder
-        # and replace inside UpsertSweep in the backend (mutation.go)
-        _sweep_id_placeholder = "WANDB_SWEEP_ID"
+        if entity is None or project is None:
+            _msg = "Must specify --entity and --project flags when using launch."
+            wandb.termerror(_msg)
+            raise LaunchError(_msg)
+
+        # Try and get job from sweep config
+        _job = config.get("job", None)
+        if _job is None:
+            wandb.termlog("No job found in sweep config, looking in launch config.")
+            _job = launch_config.get("job", None)
+            if _job is None:
+                wandb.termlog("No job found in launch config, using placeholder.")
+                _job = "placeholder-job"
 
         # Launch job spec for the Scheduler
-        # TODO: Keep up to date with Launch Job Spec
         _launch_scheduler_spec = json.dumps(
             {
-                "queue": queue,
+                "queue": launch_config.get("queue", "default"),
                 "run_spec": json.dumps(
                     construct_launch_spec(
-                        os.getcwd(),  # uri,
+                        "placeholder-uri-scheduler",  # uri
+                        None,  # job
                         api,
-                        f"Scheduler.{_sweep_id_placeholder}",  # name,
+                        "Scheduler.WANDB_SWEEP_ID",  # name,
                         project,
                         entity,
-                        None,  # docker_image,
-                        "local-process",  # resource,
+                        launch_config.get("scheduler", {}).get(
+                            "docker_image", None
+                        ),  # docker_image,
+                        launch_config.get("scheduler", {}).get(
+                            "resource", "local-process"
+                        ),  # resource,
                         [
                             "wandb",
                             "scheduler",
-                            _sweep_id_placeholder,
+                            "WANDB_SWEEP_ID",
                             "--queue",
-                            queue,
+                            launch_config.get("queue", "default"),
                             "--project",
                             project,
+                            "--job",
+                            _job,
+                            "--resource",
+                            launch_config.get("resource", "local-process"),
+                            # TODO(hupo): Add num-workers as option in launch config
+                            # "--num_workers",
+                            # launch_config.get("scheduler", {}).get("num_workers", 1),
                         ],  # entry_point,
                         None,  # version,
-                        None,  # params,
-                        None,  # resource_args,
+                        None,  # parameters,
+                        launch_config.get("scheduler", {}).get(
+                            "resource_args", None
+                        ),  # resource_args,
                         None,  # launch_config,
                         None,  # cuda,
                         None,  # run_id,
@@ -951,12 +967,8 @@ def sweep(
     if sweep_path.find(" ") >= 0:
         sweep_path = f'"{sweep_path}"'
 
-    if queue is not None:
-        wandb.termlog(
-            "If no launch agent is running, run launch agent with: {}".format(
-                click.style(f"wandb launch-agent -q {queue} -p {project}", fg="yellow")
-            )
-        )
+    if launch_config is not None:
+        wandb.termlog("Scheduler added to launch queue. Starting sweep...")
     else:
         wandb.termlog(
             "Run sweep agent with: {}".format(
@@ -1167,6 +1179,10 @@ def launch(
     elif resource is None:
         resource = "local-container"
 
+    check_logged_in(api)
+
+    run_id = config.get("run_id")
+
     if queue is None:
         # direct launch
         try:
@@ -1186,6 +1202,7 @@ def launch(
                 config=config,
                 synchronous=(not run_async),
                 cuda=cuda,
+                run_id=run_id,
             )
         except LaunchError as e:
             logger.error("=== %s ===", e)
@@ -1210,6 +1227,7 @@ def launch(
             args_dict,
             resource_args,
             cuda=cuda,
+            run_id=run_id,
         )
 
 
@@ -1267,6 +1285,8 @@ def launch_agent(
             "You must specify a project name or set WANDB_PROJECT environment variable."
         )
 
+    check_logged_in(api)
+
     wandb.termlog("Starting launch agent ✨")
     wandb_launch.create_and_run_agent(api, agent_config)
 
@@ -1296,35 +1316,13 @@ def agent(ctx, project, entity, count, sweep_id):
 
 
 @cli.command(
-    context_settings=CONTEXT, help="Run a W&B launch sweep scheduler (Experimental)"
+    context_settings=RUN_CONTEXT, help="Run a W&B launch sweep scheduler (Experimental)"
 )
 @click.pass_context
-@click.option(
-    "--project",
-    "-p",
-    default=None,
-    help="Name of the project which the agent will watch. "
-    "If passed in, will override the project value passed in using a config file.",
-)
-@click.option(
-    "--entity",
-    "-e",
-    default=None,
-    help="The entity to use. Defaults to current logged-in user",
-)
-@click.option(
-    "--queue",
-    "-q",
-    default=None,
-    help="The queue to push sweep jobs to.",
-)
 @click.argument("sweep_id")
 @display_error
 def scheduler(
     ctx,
-    project,
-    entity,
-    queue,
     sweep_id,
 ):
     api = _get_cling_api()
@@ -1336,8 +1334,16 @@ def scheduler(
     wandb.termlog("Starting a Launch Scheduler 🚀")
     from wandb.sdk.launch.sweeps import load_scheduler
 
+    # Future-proofing hack to pull any kwargs that get passed in through the CLI
+    kwargs = {}
+    for i, _arg in enumerate(ctx.args):
+        if isinstance(_arg, str) and _arg.startswith("--"):
+            kwargs[_arg[2:]] = ctx.args[i + 1]
+
     _scheduler = load_scheduler("sweep")(
-        api, entity=entity, project=project, queue=queue, sweep_id=sweep_id
+        api,
+        sweep_id=sweep_id,
+        **kwargs,
     )
     _scheduler.start()
 
@@ -1354,11 +1360,6 @@ def controller(verbose, sweep_id):
     tuner.run(verbose=verbose)
 
 
-RUN_CONTEXT = copy.copy(CONTEXT)
-RUN_CONTEXT["allow_extra_args"] = True
-RUN_CONTEXT["ignore_unknown_options"] = True
-
-
 @cli.command(context_settings=RUN_CONTEXT, name="docker-run")
 @click.pass_context
 @click.argument("docker_run_args", nargs=-1)
@@ -1741,6 +1742,8 @@ def put(path, name, description, type, alias):
     artifact_path = artifact_path.split(":")[0] + ":" + res.get("version", "latest")
     # Re-create the artifact and actually upload any files needed
     run.log_artifact(artifact, aliases=alias)
+    artifact.wait()
+
     wandb.termlog(
         "Artifact uploaded, use this artifact in a run by adding:\n", prefix=False
     )
@@ -2081,7 +2084,7 @@ def online():
     except configparser.Error:
         pass
     click.echo(
-        "W&B online, running your script from this directory will now sync to the cloud."
+        "W&B online. Running your script from this directory will now sync to the cloud."
     )
 
 
@@ -2093,11 +2096,11 @@ def offline():
         api.set_setting("disabled", "true", persist=True)
         api.set_setting("mode", "offline", persist=True)
         click.echo(
-            "W&B offline, running your script from this directory will only write metadata locally."
+            "W&B offline. Running your script from this directory will only write metadata locally. Use wandb disabled to completely turn off W&B."
         )
     except configparser.Error:
         click.echo(
-            "Unable to write config, copy and paste the following in your terminal to turn off W&B:\nexport WANDB_MODE=dryrun"
+            "Unable to write config, copy and paste the following in your terminal to turn off W&B:\nexport WANDB_MODE=offline"
         )
 
 
@@ -2149,7 +2152,7 @@ def enabled():
         click.echo("W&B enabled.")
     except configparser.Error:
         click.echo(
-            "Unable to write config, copy and paste the following in your terminal to turn off W&B:\nexport WANDB_MODE=online"
+            "Unable to write config, copy and paste the following in your terminal to turn on W&B:\nexport WANDB_MODE=online"
         )
 
 
