@@ -22,8 +22,6 @@ import json
 import logging
 import os
 import pprint
-import re
-import sys
 import tempfile
 from typing import Optional
 
@@ -32,9 +30,9 @@ from wandb import util
 
 from .sdk.data_types import _dtypes
 from .sdk.data_types.base_types.media import (
-    _numpy_arrays_to_lists,
     BatchableMedia,
     Media,
+    _numpy_arrays_to_lists,
 )
 from .sdk.data_types.base_types.wb_value import WBValue
 from .sdk.data_types.helper_types.bounding_boxes_2d import BoundingBoxes2D
@@ -165,8 +163,7 @@ class Table(Media):
     import pandas as pd
     import wandb
 
-    data = {"users": ["geoff", "juergen", "ada"],
-            "feature_01": [1, 117, 42]}
+    data = {"users": ["geoff", "juergen", "ada"], "feature_01": [1, 117, 42]}
     df = pd.DataFrame(data)
 
     tbl = wandb.Table(data=df)
@@ -188,7 +185,11 @@ class Table(Media):
     [tbl.add_data(user) for user in users]
     assert tbl.get_column("user") == users
 
-    def get_user_name_length(index, row): return {"feature_01": len(row["user"])}
+
+    def get_user_name_length(index, row):
+        return {"feature_01": len(row["user"])}
+
+
     tbl.add_computed_columns(get_user_name_length)
     assert tbl.get_column("feature_01") == [5, 7, 3]
     ```
@@ -289,8 +290,6 @@ class Table(Media):
     @staticmethod
     def _assert_valid_columns(columns):
         valid_col_types = [str, int]
-        if sys.version_info.major < 3:
-            valid_col_types.append(unicode)  # noqa: F821 (unicode is in py2)
         assert type(columns) is list, "columns argument expects a `list` object"
         assert len(columns) == 0 or all(
             [type(col) in valid_col_types for col in columns]
@@ -597,7 +596,7 @@ class Table(Media):
         dtypes = None
         if column_types is not None:
             dtypes = [
-                column_types.params["type_map"][col] for col in json_obj["columns"]
+                column_types.params["type_map"][str(col)] for col in json_obj["columns"]
             ]
 
         new_obj = cls(columns=json_obj["columns"], data=data, dtype=dtypes)
@@ -1046,7 +1045,7 @@ class Audio(BatchableMedia):
         self._caption = caption
 
         if isinstance(data_or_path, str):
-            if Audio.path_is_reference(data_or_path):
+            if self.path_is_reference(data_or_path):
                 self._path = data_or_path
                 self._sha256 = hashlib.sha256(data_or_path.encode("utf-8")).hexdigest()
                 self._is_tmp = False
@@ -1069,10 +1068,6 @@ class Audio(BatchableMedia):
 
             self._set_file(tmp_path, is_tmp=True)
 
-    @classmethod
-    def path_is_reference(cls, path):
-        return bool(re.match(r"^(gs|s3|https?)://", path))
-
     @classmethod
     def get_media_subdir(cls):
         return os.path.join("media", "audio")
@@ -1087,7 +1082,7 @@ class Audio(BatchableMedia):
     def bind_to_run(
         self, run, key, step, id_=None, ignore_copy_err: Optional[bool] = None
     ):
-        if Audio.path_is_reference(self._path):
+        if self.path_is_reference(self._path):
             raise ValueError(
                 "Audio media created by a reference to external storage cannot currently be added to a run"
             )
@@ -1148,7 +1143,7 @@ class Audio(BatchableMedia):
             return ["" if c is None else c for c in captions]
 
     def resolve_ref(self):
-        if Audio.path_is_reference(self._path):
+        if self.path_is_reference(self._path):
             # this object was already created using a ref:
             return self._path
         source_artifact = self._artifact_source.artifact
@@ -1162,7 +1157,7 @@ class Audio(BatchableMedia):
         return None
 
     def __eq__(self, other):
-        if Audio.path_is_reference(self._path) or Audio.path_is_reference(other._path):
+        if self.path_is_reference(self._path) or self.path_is_reference(other._path):
             # one or more of these objects is an unresolved reference -- we'll compare
             # their reference paths instead of their SHAs:
             return (
