@@ -2,23 +2,22 @@
 keras init
 """
 
-import shutil
 import logging
-import numpy as np
 import operator
 import os
+import shutil
 import sys
-
 from itertools import chain
 
+import numpy as np
+import tensorflow as tf
+import tensorflow.keras.backend as K
+
 import wandb
 from wandb.sdk.integration_utils.data_logging import ValidationDataLogger
-from wandb.sdk.lib.deprecate import deprecate, Deprecated
+from wandb.sdk.lib.deprecate import Deprecated, deprecate
 from wandb.util import add_import_hook
 
-import tensorflow as tf
-import tensorflow.keras.backend as K
-
 
 def _check_keras_version():
     from keras import __version__ as keras_version
@@ -105,8 +104,10 @@ def patch_tf_keras():
             )
         except (ImportError, AttributeError):
             try:
-                from tensorflow.python.keras.engine import training_arrays
-                from tensorflow.python.keras.engine import training_generator
+                from tensorflow.python.keras.engine import (
+                    training_arrays,
+                    training_generator,
+                )
             except (ImportError, AttributeError):
                 wandb.termerror("Unable to patch Tensorflow/Keras")
                 logger.exception("exception while trying to patch_tf_keras")
@@ -368,6 +369,7 @@ class WandbCallback(tf.keras.callbacks.Callback):
             processors where appropriate.
         log_evaluation_frequency: (int) Determines the frequency which evaluation results will be logged. Default 0 (only at the end of training).
             Set to 1 to log every epoch, 2 to log every other epoch, and so on. Has no effect when log_evaluation is False.
+        compute_flops: (bool) Compute the FLOPs of your Keras Sequential or Functional model in GigaFLOPs unit.
     """
 
     def __init__(
@@ -397,6 +399,7 @@ class WandbCallback(tf.keras.callbacks.Callback):
         prediction_row_processor=None,
         infer_missing_processors=True,
         log_evaluation_frequency=0,
+        compute_flops=False,
         **kwargs,
     ):
         if wandb.run is None:
@@ -458,6 +461,7 @@ class WandbCallback(tf.keras.callbacks.Callback):
         self.class_colors = np.array(class_colors) if class_colors is not None else None
         self.log_batch_frequency = log_batch_frequency
         self.log_best_prefix = log_best_prefix
+        self.compute_flops = compute_flops
 
         self._prediction_batch_size = None
 
@@ -698,7 +702,7 @@ class WandbCallback(tf.keras.callbacks.Callback):
                     f"Skipping logging validation data. Error: {str(e)}"
                 )
 
-        if _can_compute_flops():
+        if self.compute_flops and _can_compute_flops():
             try:
                 wandb.summary["GFLOPs"] = self.get_flops()
             except Exception as e:
@@ -1067,7 +1071,5 @@ class WandbCallback(tf.keras.callbacks.Callback):
             graph=frozen_func.graph, run_meta=run_meta, cmd="scope", options=opts
         )
 
-        tf.compat.v1.reset_default_graph()
-
         # convert to GFLOPs
         return (flops.total_float_ops / 1e9) / 2
