@@ -4,6 +4,7 @@ import textwrap
 from typing import Callable, List, Mapping, Optional
 
 from pkg_resources import parse_version
+
 import wandb
 
 try:
@@ -96,9 +97,18 @@ def unpatch_kfp():
 
 def patch_kfp():
     to_patch = [
-        ("kfp.components", create_component_from_func,),
-        ("kfp.components._python_op", create_component_from_func,),
-        ("kfp.components._python_op", _get_function_source_definition,),
+        (
+            "kfp.components",
+            create_component_from_func,
+        ),
+        (
+            "kfp.components._python_op",
+            create_component_from_func,
+        ),
+        (
+            "kfp.components._python_op",
+            _get_function_source_definition,
+        ),
         ("kfp.components._python_op", strip_type_hints),
     ]
 
@@ -281,7 +291,9 @@ def create_component_from_func(
         packages_to_install=packages_to_install,
     )
     if annotations:
-        component_spec.metadata = structures.MetadataSpec(annotations=annotations,)
+        component_spec.metadata = structures.MetadataSpec(
+            annotations=annotations,
+        )
 
     if output_component_file:
         component_spec.save(output_component_file)
