@@ -3,8 +3,7 @@ def wandb_log(  # noqa: C901
     # /,  # py38 only
     log_component_file=True,
 ):
-    """Wrap a standard python function and log to W&B
-    """
+    """Wrap a standard python function and log to W&B"""
     import json
     import os
     from functools import wraps
@@ -21,8 +20,8 @@ def wandb_log(  # noqa: C901
         OutputPath,
         OutputTextFile,
     )
-    import wandb
 
+    import wandb
     from wandb.sdk.lib import telemetry as wb_telemetry
 
     output_types = (OutputArtifact, OutputBinaryFile, OutputPath, OutputTextFile)
