@@ -1,15 +1,16 @@
 import io
 import re
 import time
-from typing import Any, Dict, List, Optional, TYPE_CHECKING, Union
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Union
 
 import wandb
-from wandb.sdk.lib import telemetry
 import wandb.util
-from wandb.viz import create_custom_chart
+from wandb.sdk.lib import telemetry
+from wandb.viz import custom_chart
 
 if TYPE_CHECKING:
     import numpy as np  # type: ignore
+
     from wandb.sdk.internal.tb_watcher import TBHistory
 
 # We have at least the default namestep and a global step to track
@@ -201,7 +202,8 @@ def tf_summary_to_dict(  # noqa: C901
                 data = sorted(data, key=lambda x: (x[0], -x[1]))
                 data_table = wandb.Table(data=data, columns=["recall", "precision"])
                 name = namespaced_tag(value.tag, namespace)
-                values[name] = create_custom_chart(
+
+                values[name] = custom_chart(
                     "wandb/line/v0",
                     data_table,
                     {"x": "recall", "y": "precision"},
