@@ -1,31 +1,22 @@
-from __future__ import unicode_literals
-
 import json
 import os
-import sys
 import time
-import requests
 
 from wandb_gql import gql
-import six
 
 import wandb
 from wandb import util
-from wandb.sdk.data_types.utils import val_to_json
 from wandb.apis.internal import Api
-from six import string_types
-
-
 from wandb.sdk import lib as wandb_lib
+from wandb.sdk.data_types.utils import val_to_json
 
-
-DEEP_SUMMARY_FNAME = 'wandb.h5'
+DEEP_SUMMARY_FNAME = "wandb.h5"
 H5_TYPES = ("numpy.ndarray", "tensorflow.Tensor", "torch.Tensor")
 h5py = util.get_module("h5py")
 np = util.get_module("numpy")
 
 
-class SummarySubDict(object):
+class SummarySubDict:
     """Nested dict-like object that proxies read and write operations through a root object.
 
     This lets us do synchronous serialization and lazy loading of large values.
@@ -98,7 +89,7 @@ class SummarySubDict(object):
         return self._json_dict.keys()
 
     def get(self, k, default=None):
-        if isinstance(k, string_types):
+        if isinstance(k, str):
             k = k.strip()
         if k not in self._dict:
             self._root._root_get(self._path + (k,), self._dict)
@@ -111,7 +102,7 @@ class SummarySubDict(object):
             yield k, self[k]
 
     def __getitem__(self, k):
-        if isinstance(k, string_types):
+        if isinstance(k, str):
             k = k.strip()
 
         self.get(k)  # load the value into _dict if it should be there
@@ -120,13 +111,13 @@ class SummarySubDict(object):
         return res
 
     def __contains__(self, k):
-        if isinstance(k, string_types):
+        if isinstance(k, str):
             k = k.strip()
 
         return k in self._json_dict
 
     def __setitem__(self, k, v):
-        if isinstance(k, string_types):
+        if isinstance(k, str):
             k = k.strip()
 
         path = self._path
@@ -158,10 +149,14 @@ class SummarySubDict(object):
         repr_dict = dict(self._dict)
         for k in self._json_dict:
             v = self._json_dict[k]
-            if k not in repr_dict and isinstance(v, dict) and v.get('_type') in H5_TYPES:
+            if (
+                k not in repr_dict
+                and isinstance(v, dict)
+                and v.get("_type") in H5_TYPES
+            ):
                 # unloaded h5 objects may be very large. use a placeholder for them
                 # if we haven't already loaded them
-                repr_dict[k] = '...'
+                repr_dict[k] = "..."
             else:
                 repr_dict[k] = self[k]
 
@@ -181,10 +176,10 @@ class SummarySubDict(object):
         if not key_vals:
             return
 
-        key_vals = dict((k.strip(), v) for k, v in six.iteritems(key_vals))
+        key_vals = {k.strip(): v for k, v in key_vals.items()}
 
         if overwrite:
-            write_items = list(six.iteritems(key_vals))
+            write_items = list(key_vals.items())
             self._locked_keys.update(key_vals.keys())
         else:
             write_keys = set(key_vals.keys()) - self._locked_keys
@@ -192,8 +187,7 @@ class SummarySubDict(object):
 
         for key, value in write_items:
             if isinstance(value, dict):
-                self._dict[key] = SummarySubDict(
-                    self._root, self._path + (key,))
+                self._dict[key] = SummarySubDict(self._root, self._path + (key,))
                 self._dict[key]._update(value, overwrite)
             else:
                 self._dict[key] = value
@@ -210,7 +204,7 @@ class Summary(SummarySubDict):
     """
 
     def __init__(self, run, summary=None):
-        super(Summary, self).__init__()
+        super().__init__()
         self._run = run
         self._h5_path = os.path.join(self._run.dir, DEEP_SUMMARY_FNAME)
         # Lazy load the h5 file
@@ -247,7 +241,7 @@ class Summary(SummarySubDict):
                 wandb.termerror("Deleting tensors in summary requires h5py")
             else:
                 self.open_h5()
-                h5_key = "summary/" + '.'.join(path)
+                h5_key = "summary/" + ".".join(path)
                 del self._h5[h5_key]
                 self._h5.flush()
 
@@ -267,10 +261,10 @@ class Summary(SummarySubDict):
             wandb.termerror("Storing tensors in summary requires h5py")
         else:
             try:
-                del self._h5["summary/" + '.'.join(path)]
+                del self._h5["summary/" + ".".join(path)]
             except KeyError:
                 pass
-            self._h5["summary/" + '.'.join(path)] = val
+            self._h5["summary/" + ".".join(path)] = val
             self._h5.flush()
 
     def read_h5(self, path, val=None):
@@ -280,11 +274,11 @@ class Summary(SummarySubDict):
         if not self._h5:
             wandb.termerror("Reading tensors from summary requires h5py")
         else:
-            return self._h5.get("summary/" + '.'.join(path), val)
+            return self._h5.get("summary/" + ".".join(path), val)
 
     def open_h5(self):
         if not self._h5 and h5py:
-            self._h5 = h5py.File(self._h5_path, 'a', libver='latest')
+            self._h5 = h5py.File(self._h5_path, "a", libver="latest")
 
     def _decode(self, path, json_value):
         """Decode a `dict` encoded by `Summary._encode()`, loading h5 objects.
@@ -294,9 +288,10 @@ class Summary(SummarySubDict):
         if isinstance(json_value, dict):
             if json_value.get("_type") in H5_TYPES:
                 return self.read_h5(path, json_value)
-            elif json_value.get("_type") == 'data-frame':
+            elif json_value.get("_type") == "data-frame":
                 wandb.termerror(
-                    'This data frame was saved via the wandb data API. Contact support@wandb.com for help.')
+                    "This data frame was saved via the wandb data API. Contact support@wandb.com for help."
+                )
                 return None
             # TODO: transform wandb objects and plots
             else:
@@ -321,15 +316,17 @@ class Summary(SummarySubDict):
 
         if isinstance(value, dict):
             json_value = {}
-            for key, value in six.iteritems(value):
+            for key, value in value.items():
                 json_value[key] = self._encode(value, path_from_root + (key,))
             return json_value
         else:
             path = ".".join(path_from_root)
             friendly_value, converted = util.json_friendly(
-                val_to_json(self._run, path, value, namespace="summary"))
+                val_to_json(self._run, path, value, namespace="summary")
+            )
             json_value, compressed = util.maybe_compress_summary(
-                friendly_value, util.get_h5_typename(value))
+                friendly_value, util.get_h5_typename(value)
+            )
             if compressed:
                 self.write_h5(path_from_root, friendly_value)
 
@@ -338,9 +335,13 @@ class Summary(SummarySubDict):
 
 def download_h5(run_id, entity=None, project=None, out_dir=None):
     api = Api()
-    meta = api.download_url(project or api.settings(
-        "project"), DEEP_SUMMARY_FNAME, entity=entity or api.settings("entity"), run=run_id)
-    if meta and 'md5' in meta and meta['md5'] is not None:
+    meta = api.download_url(
+        project or api.settings("project"),
+        DEEP_SUMMARY_FNAME,
+        entity=entity or api.settings("entity"),
+        run=run_id,
+    )
+    if meta and "md5" in meta and meta["md5"] is not None:
         # TODO: make this non-blocking
         wandb.termlog("Downloading summary data...")
         path, res = api.download_write_file(meta, out_dir=out_dir)
@@ -350,14 +351,15 @@ def download_h5(run_id, entity=None, project=None, out_dir=None):
 def upload_h5(file, run_id, entity=None, project=None):
     api = Api()
     wandb.termlog("Uploading summary data...")
-    with open(file, 'rb') as f:
-        api.push({os.path.basename(file): f}, run=run_id, project=project,
-                entity=entity)
+    with open(file, "rb") as f:
+        api.push(
+            {os.path.basename(file): f}, run=run_id, project=project, entity=entity
+        )
 
 
 class FileSummary(Summary):
     def __init__(self, run):
-        super(FileSummary, self).__init__(run)
+        super().__init__(run)
         self._fname = os.path.join(run.dir, wandb_lib.filenames.SUMMARY_FNAME)
         self.load()
 
@@ -365,14 +367,14 @@ class FileSummary(Summary):
         try:
             with open(self._fname) as f:
                 self._json_dict = json.load(f)
-        except (IOError, ValueError):
+        except (OSError, ValueError):
             self._json_dict = {}
 
     def _write(self, commit=False):
         # TODO: we just ignore commit to ensure backward capability
-        with open(self._fname, 'w') as f:
+        with open(self._fname, "w") as f:
             f.write(util.json_dumps_safer(self._json_dict))
-            f.write('\n')
+            f.write("\n")
             f.flush()
             os.fsync(f.fileno())
         if self._h5:
@@ -384,7 +386,7 @@ class FileSummary(Summary):
 
 class HTTPSummary(Summary):
     def __init__(self, run, client, summary=None):
-        super(HTTPSummary, self).__init__(run, summary=summary)
+        super().__init__(run, summary=summary)
         self._run = run
         self._client = client
         self._started = time.time()
@@ -394,26 +396,41 @@ class HTTPSummary(Summary):
 
     def open_h5(self):
         if not self._h5 and h5py:
-            download_h5(self._run.id, entity=self._run.entity, project=self._run.project, out_dir=self._run.dir)
-        super(HTTPSummary, self).open_h5()
+            download_h5(
+                self._run.id,
+                entity=self._run.entity,
+                project=self._run.project,
+                out_dir=self._run.dir,
+            )
+        super().open_h5()
 
     def _write(self, commit=False):
-        mutation = gql('''
+        mutation = gql(
+            """
         mutation UpsertBucket( $id: String, $summaryMetrics: JSONString) {
             upsertBucket(input: { id: $id, summaryMetrics: $summaryMetrics}) {
                 bucket { id }
             }
         }
-        ''')
+        """
+        )
         if commit:
             if self._h5:
                 self._h5.close()
                 self._h5 = None
-            res = self._client.execute(mutation, variable_values={
-                'id': self._run.storage_id, 'summaryMetrics': util.json_dumps_safer(self._json_dict)})
-            assert res['upsertBucket']['bucket']['id']
+            res = self._client.execute(
+                mutation,
+                variable_values={
+                    "id": self._run.storage_id,
+                    "summaryMetrics": util.json_dumps_safer(self._json_dict),
+                },
+            )
+            assert res["upsertBucket"]["bucket"]["id"]
             entity, project, run = self._run.path
-            if os.path.exists(self._h5_path) and os.path.getmtime(self._h5_path) >= self._started:
+            if (
+                os.path.exists(self._h5_path)
+                and os.path.getmtime(self._h5_path) >= self._started
+            ):
                 upload_h5(self._h5_path, run, entity=entity, project=project)
         else:
             return False
