@@ -1,10 +1,10 @@
 import hashlib
-from io import BytesIO
 import logging
 import os
-from typing import Any, cast, Dict, List, Optional, Sequence, Type, TYPE_CHECKING, Union
+from io import BytesIO
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Sequence, Type, Union, cast
+from urllib import parse
 
-from pkg_resources import parse_version
 import wandb
 from wandb import util
 
@@ -19,6 +19,7 @@ if TYPE_CHECKING:  # pragma: no cover
     import numpy as np  # type: ignore
     import PIL  # type: ignore
     import torch  # type: ignore
+
     from wandb.apis.public import Artifact as PublicArtifact
 
     from ..wandb_artifacts import Artifact as LocalArtifact
@@ -32,20 +33,28 @@ if TYPE_CHECKING:  # pragma: no cover
 
 
 def _server_accepts_image_filenames() -> bool:
+    from pkg_resources import parse_version
+
     # Newer versions of wandb accept large image filenames arrays
     # but older versions would have issues with this.
     max_cli_version = util._get_max_cli_version()
     if max_cli_version is None:
         return False
-    return parse_version("0.12.10") <= parse_version(max_cli_version)
+    accepts_image_filenames: bool = parse_version("0.12.10") <= parse_version(
+        max_cli_version
+    )
+    return accepts_image_filenames
 
 
 def _server_accepts_artifact_path() -> bool:
+    from pkg_resources import parse_version
+
     target_version = "0.12.14"
     max_cli_version = util._get_max_cli_version() if not util._is_offline() else None
-    return max_cli_version is not None and parse_version(
+    accepts_artifact_path: bool = max_cli_version is not None and parse_version(
         target_version
     ) <= parse_version(max_cli_version)
+    return accepts_artifact_path
 
 
 class Image(BatchableMedia):
@@ -140,7 +149,10 @@ class Image(BatchableMedia):
         if isinstance(data_or_path, Image):
             self._initialize_from_wbimage(data_or_path)
         elif isinstance(data_or_path, str):
-            self._initialize_from_path(data_or_path)
+            if self.path_is_reference(data_or_path):
+                self._initialize_from_reference(data_or_path)
+            else:
+                self._initialize_from_path(data_or_path)
         else:
             self._initialize_from_data(data_or_path, mode)
 
@@ -206,7 +218,8 @@ class Image(BatchableMedia):
                     for key in total_classes.keys()
                 ]
             )
-        self._width, self._height = self.image.size  # type: ignore
+        if self.image:
+            self._width, self._height = self.image.size
         self._free_ram()
 
     def _initialize_from_wbimage(self, wbimage: "Image") -> None:
@@ -240,6 +253,14 @@ class Image(BatchableMedia):
         ext = os.path.splitext(path)[1][1:]
         self.format = ext
 
+    def _initialize_from_reference(self, path: str) -> None:
+        self._path = path
+        self._is_tmp = False
+        self._sha256 = hashlib.sha256(path.encode("utf-8")).hexdigest()
+        path = parse.urlparse(path).path
+        ext = path.split("/")[-1].split(".")[-1]
+        self.format = ext
+
     def _initialize_from_data(
         self,
         data: "ImageDataType",
@@ -334,6 +355,11 @@ class Image(BatchableMedia):
         # space, but there are also custom charts, and maybe others. Let's
         # commit to getting all that fixed up before moving this to  the top
         # level Media class.
+        if self.path_is_reference(self._path):
+            raise ValueError(
+                "Image media created by a reference to external storage cannot currently be added to a run"
+            )
+
         if (
             not _server_accepts_artifact_path()
             or self._get_artifact_entry_ref_url() is None
@@ -580,6 +606,10 @@ class Image(BatchableMedia):
         if not isinstance(other, Image):
             return False
         else:
+            if self.path_is_reference(self._path) and self.path_is_reference(
+                other._path
+            ):
+                return self._path == other._path
             self_image = self.image
             other_image = other.image
             if self_image is not None:
@@ -612,7 +642,7 @@ class Image(BatchableMedia):
     @property
     def image(self) -> Optional["PIL.Image"]:
         if self._image is None:
-            if self._path is not None:
+            if self._path is not None and not self.path_is_reference(self._path):
                 pil_image = util.get_module(
                     "PIL.Image",
                     required='wandb.Image needs the PIL package. To get it, run "pip install pillow".',
