@@ -6,27 +6,41 @@ import hashlib
 import os
 import random
 from typing import (
-    Callable,
+    IO,
+    TYPE_CHECKING,
+    ContextManager,
     Dict,
     List,
     Optional,
     Sequence,
     Tuple,
-    TYPE_CHECKING,
     Union,
 )
 
 import wandb
-from wandb import env
-from wandb import util
+from wandb import env, util
 from wandb.data_types import WBValue
 
-
 if TYPE_CHECKING:
-    import wandb.filesync.step_prepare.StepPrepare as StepPrepare  # type: ignore
+    # need this import for type annotations, but want to avoid circular dependency
+    import sys
+
+    import wandb.apis.public
+    from wandb.filesync.step_prepare import StepPrepare
+    from wandb.sdk import wandb_artifacts
+    from wandb.sdk.internal import progress
+
+    if sys.version_info >= (3, 8):
+        from typing import Protocol
+    else:
+        from typing_extensions import Protocol
+
+    class Opener(Protocol):
+        def __call__(self, mode: str = ...) -> ContextManager[IO]:
+            pass
 
 
-def md5_string(string: str) -> str:
+def md5_string(string: str) -> util.B64MD5:
     hash_md5 = hashlib.md5()
     hash_md5.update(string.encode())
     return base64.b64encode(hash_md5.digest()).decode("ascii")
@@ -36,7 +50,7 @@ def b64_string_to_hex(string):
     return binascii.hexlify(base64.standard_b64decode(string)).decode("ascii")
 
 
-def md5_hash_file(path):
+def md5_hash_file(path) -> util.RawMD5:
     hash_md5 = hashlib.md5()
     with open(path, "rb") as f:
         for chunk in iter(lambda: f.read(64 * 1024), b""):
@@ -44,7 +58,7 @@ def md5_hash_file(path):
     return hash_md5
 
 
-def md5_hash_files(paths: List[str]):
+def md5_hash_files(paths: List[str]) -> util.RawMD5:
     hash_md5 = hashlib.md5()
     # Create a mutable copy to sort
     paths = [path for path in paths]
@@ -56,15 +70,15 @@ def md5_hash_files(paths: List[str]):
     return hash_md5
 
 
-def md5_file_b64(path: str) -> str:
+def md5_file_b64(path: str) -> util.B64MD5:
     return base64.b64encode(md5_hash_file(path).digest()).decode("ascii")
 
 
-def md5_files_b64(paths: List[str]) -> str:
+def md5_files_b64(paths: List[str]) -> util.B64MD5:
     return base64.b64encode(md5_hash_files(paths).digest()).decode("ascii")
 
 
-def md5_file_hex(path: str) -> str:
+def md5_file_hex(path: str) -> util.HexMD5:
     return md5_hash_file(path).hexdigest()
 
 
@@ -91,7 +105,12 @@ class ArtifactManifest:
     def version(cls):
         pass
 
-    def __init__(self, artifact, storage_policy, entries=None):
+    def __init__(
+        self,
+        artifact,
+        storage_policy: "wandb_artifacts.WandbStoragePolicy",
+        entries=None,
+    ) -> None:
         self.artifact = artifact
         self.storage_policy = storage_policy
         self.entries = entries or {}
@@ -124,9 +143,9 @@ class ArtifactManifest:
 
 
 class ArtifactEntry:
-    path: str
-    ref: Optional[str]
-    digest: str
+    path: util.LogicalFilePathStr
+    ref: Optional[Union[util.FilePathStr, util.URIStr]]
+    digest: Union[util.B64MD5, util.URIStr, util.FilePathStr, util.ETag]
     birth_artifact_id: Optional[str]
     size: Optional[int]
     extra: Dict
@@ -141,7 +160,7 @@ class ArtifactEntry:
         """
         raise NotImplementedError
 
-    def download(self, root: Optional[str] = None) -> str:
+    def download(self, root: Optional[str] = None) -> util.FilePathStr:
         """
         Downloads this artifact entry to the specified root path.
 
@@ -196,7 +215,7 @@ class Artifact:
     def version(self) -> str:
         """
         Returns:
-            (int): The version of this artifact. For example, if this
+            (str): The version of this artifact. For example, if this
                 is the first version of an artifact, its `version` will
                 be 'v0'.
         """
@@ -601,7 +620,9 @@ class Artifact:
         """
         raise NotImplementedError
 
-    def download(self, root: Optional[str] = None, recursive: bool = False) -> str:
+    def download(
+        self, root: Optional[str] = None, recursive: bool = False
+    ) -> util.FilePathStr:
         """
         Downloads the contents of the artifact to the specified root directory.
 
@@ -794,7 +815,7 @@ class StoragePolicy:
         artifact_manifest_id: str,
         entry: ArtifactEntry,
         preparer: "StepPrepare",
-        progress_callback: Optional[Callable] = None,
+        progress_callback: Optional["progress.ProgressFn"] = None,
     ) -> bool:
         raise NotImplementedError
 
@@ -827,7 +848,7 @@ class StorageHandler:
         artifact: Artifact,
         manifest_entry: ArtifactEntry,
         local: bool = False,
-    ) -> str:
+    ) -> Union[util.URIStr, util.FilePathStr]:
         """
         Loads the file or directory within the specified artifact given its
         corresponding index entry.
@@ -869,7 +890,9 @@ class ArtifactsCache:
         self._random.seed()
         self._artifacts_by_client_id = {}
 
-    def check_md5_obj_path(self, b64_md5: str, size: int) -> Tuple[str, bool, Callable]:
+    def check_md5_obj_path(
+        self, b64_md5: util.B64MD5, size: int
+    ) -> Tuple[util.FilePathStr, bool, "Opener"]:
         hex_md5 = util.bytes_to_hex(base64.b64decode(b64_md5))
         path = os.path.join(self._cache_dir, "obj", "md5", hex_md5[:2], hex_md5[2:])
         opener = self._cache_opener(path)
@@ -878,8 +901,19 @@ class ArtifactsCache:
         util.mkdir_exists_ok(os.path.dirname(path))
         return path, False, opener
 
-    def check_etag_obj_path(self, etag: str, size: int) -> Tuple[str, bool, Callable]:
-        path = os.path.join(self._cache_dir, "obj", "etag", etag[:2], etag[2:])
+    # TODO(spencerpearson): this method at least needs its signature changed.
+    # An ETag is not (necessarily) a checksum.
+    def check_etag_obj_path(
+        self,
+        url: util.URIStr,
+        etag: util.ETag,
+        size: int,
+    ) -> Tuple[util.FilePathStr, bool, "Opener"]:
+        hexhash = hashlib.sha256(
+            hashlib.sha256(url.encode("utf-8")).digest()
+            + hashlib.sha256(etag.encode("utf-8")).digest()
+        ).hexdigest()
+        path = os.path.join(self._cache_dir, "obj", "etag", hexhash[:2], hexhash[2:])
         opener = self._cache_opener(path)
         if os.path.isfile(path) and os.path.getsize(path) == size:
             return path, True, opener
@@ -935,6 +969,10 @@ class ArtifactsCache:
     def _cache_opener(self, path):
         @contextlib.contextmanager
         def helper(mode="w"):
+
+            if "a" in mode:
+                raise ValueError("Appending to cache files is not supported")
+
             dirname = os.path.dirname(path)
             tmp_file = os.path.join(
                 dirname,
