@@ -1,17 +1,17 @@
-"""MesssageFuture - represents a message result of an asynchronous operation
+"""MessageFuture - represents a message result of an asynchronous operation
 
 Base class MessageFuture for MessageFutureObject and MessageFuturePoll
 
 """
 
-from abc import abstractmethod
 import threading
+from abc import abstractmethod
 from typing import Optional
 
 from wandb.proto import wandb_internal_pb2 as pb
 
 
-class MessageFuture(object):
+class MessageFuture:
     _object: Optional[pb.Result]
 
     def __init__(self) -> None:
