@@ -5,14 +5,15 @@ Router to manage responses from a queue.
 """
 
 import queue
-from typing import Optional
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Optional
 
-from .router import MessageRouter
 from ..lib import tracelog
+from ..lib.mailbox import Mailbox
+from .router import MessageRouter
 
 if TYPE_CHECKING:
     from queue import Queue
+
     from wandb.proto import wandb_internal_pb2 as pb
 
 
@@ -21,11 +22,14 @@ class MessageQueueRouter(MessageRouter):
     _response_queue: "Queue[pb.Result]"
 
     def __init__(
-        self, request_queue: "Queue[pb.Record]", response_queue: "Queue[pb.Result]"
+        self,
+        request_queue: "Queue[pb.Record]",
+        response_queue: "Queue[pb.Result]",
+        mailbox: Optional[Mailbox] = None,
     ) -> None:
         self._request_queue = request_queue
         self._response_queue = response_queue
-        super().__init__()
+        super().__init__(mailbox=mailbox)
 
     def _read_message(self) -> "Optional[pb.Result]":
         try:
