@@ -1,24 +1,24 @@
 """Handle Manager."""
 
-from collections import defaultdict
 import json
 import logging
 import math
 import numbers
+import time
+from collections import defaultdict
 from queue import Queue
 from threading import Event
-import time
 from typing import (
+    TYPE_CHECKING,
     Any,
     Callable,
-    cast,
     Dict,
     Iterable,
     List,
     Optional,
     Sequence,
     Tuple,
-    TYPE_CHECKING,
+    cast,
 )
 
 from wandb.proto.wandb_internal_pb2 import (
@@ -31,16 +31,14 @@ from wandb.proto.wandb_internal_pb2 import (
     SummaryRecord,
 )
 
-from . import meta, sample, stats, tb_watcher
-from .settings_static import SettingsStatic
 from ..interface.interface_queue import InterfaceQueue
 from ..lib import handler_util, proto_util, tracelog
+from . import sample, tb_watcher
+from .settings_static import SettingsStatic
+from .system.system_monitor import SystemMonitor
 
 if TYPE_CHECKING:
-    from wandb.proto.wandb_internal_pb2 import (
-        ArtifactDoneRequest,
-        MetricSummary,
-    )
+    from wandb.proto.wandb_internal_pb2 import ArtifactDoneRequest, MetricSummary
 
 
 SummaryDict = Dict[str, Any]
@@ -72,7 +70,7 @@ class HandleManager:
     _sender_q: "Queue[Record]"
     _writer_q: "Queue[Record]"
     _interface: InterfaceQueue
-    _system_stats: Optional[stats.SystemStats]
+    _system_monitor: Optional[SystemMonitor]
     _tb_watcher: Optional[tb_watcher.TBWatcher]
     _metric_defines: Dict[str, MetricRecord]
     _metric_globs: Dict[str, MetricRecord]
@@ -102,7 +100,7 @@ class HandleManager:
         self._interface = interface
 
         self._tb_watcher = None
-        self._system_stats = None
+        self._system_monitor = None
         self._step = 0
 
         self._track_time = None
@@ -128,7 +126,7 @@ class HandleManager:
         record_type = record.WhichOneof("record_type")
         assert record_type
         handler_str = "handle_" + record_type
-        handler: Callable[[Record], None] = getattr(self, handler_str, None)
+        handler: Callable[[Record], None] = getattr(self, handler_str, None)  # type: ignore
         assert handler, f"unknown handle: {handler_str}"
         handler(record)
 
@@ -136,7 +134,7 @@ class HandleManager:
         request_type = record.request.WhichOneof("request_type")
         assert request_type
         handler_str = "handle_request_" + request_type
-        handler: Callable[[Record], None] = getattr(self, handler_str, None)
+        handler: Callable[[Record], None] = getattr(self, handler_str, None)  # type: ignore
         if request_type != "network_status":
             logger.debug(f"handle_request: {request_type}")
         assert handler, f"unknown handle: {handler_str}"
@@ -164,10 +162,10 @@ class HandleManager:
         logger.info(f"handle defer: {state}")
         # only handle flush tb (sender handles the rest)
         if state == defer.FLUSH_STATS:
-            if self._system_stats:
-                # TODO(jhr): this could block so we dont really want to call shutdown
-                # from handler thread
-                self._system_stats.shutdown()
+            # TODO(jhr): this could block so we dont really want to call shutdown
+            # from handler thread
+            if self._system_monitor is not None:
+                self._system_monitor.finish()
         elif state == defer.FLUSH_TB:
             if self._tb_watcher:
                 # shutdown tensorboard workers so we get all metrics flushed
@@ -665,16 +663,15 @@ class HandleManager:
         else:
             self._accumulate_time = 0
 
+        # system monitor
+        self._system_monitor = SystemMonitor(
+            self._settings,
+            self._interface,
+        )
         if not self._settings._disable_stats:
-            self._system_stats = stats.SystemStats(
-                settings=self._settings, interface=self._interface
-            )
-            self._system_stats.start()
-
+            self._system_monitor.start()
         if not self._settings._disable_meta and not run_start.run.resumed:
-            run_meta = meta.Meta(settings=self._settings, interface=self._interface)
-            run_meta.probe()
-            run_meta.write()
+            self._system_monitor.probe(publish=True)
 
         self._tb_watcher = tb_watcher.TBWatcher(
             self._settings, interface=self._interface, run_proto=run_start.run
@@ -686,18 +683,18 @@ class HandleManager:
         self._respond_result(result)
 
     def handle_request_resume(self, record: Record) -> None:
-        if self._system_stats is not None:
-            logger.info("starting system metrics thread")
-            self._system_stats.start()
+        if self._system_monitor is not None:
+            logger.info("starting system metrics thread or process")
+            self._system_monitor.start()
 
         if self._track_time is not None:
             self._accumulate_time += time.time() - self._track_time
         self._track_time = time.time()
 
     def handle_request_pause(self, record: Record) -> None:
-        if self._system_stats is not None:
-            logger.info("stopping system metrics thread")
-            self._system_stats.shutdown()
+        if self._system_monitor is not None:
+            logger.info("stopping system metrics thread or process")
+            self._system_monitor.finish()
         if self._track_time is not None:
             self._accumulate_time += time.time() - self._track_time
             self._track_time = None
@@ -800,6 +797,13 @@ class HandleManager:
             result.response.sampled_history_response.item.append(item)
         self._respond_result(result)
 
+    def handle_request_server_info(self, record: Record) -> None:
+        self._dispatch_record(record, always_send=True)
+
+    def handle_request_keepalive(self, record: Record) -> None:
+        """keepalive is a noop, we just want to verify transport is alive."""
+        pass
+
     def handle_request_shutdown(self, record: Record) -> None:
         # TODO(jhr): should we drain things and stop new requests from coming in?
         result = proto_util._result_from_record(record)
