@@ -12,16 +12,15 @@ import threading
 import time
 from typing import TYPE_CHECKING
 
-
 from ..lib import tracelog
 
-
 if TYPE_CHECKING:
-    from typing import Tuple, Type, Optional, Union
     from queue import Queue
-    from wandb.proto.wandb_internal_pb2 import Record, Result
     from threading import Event
     from types import TracebackType
+    from typing import Optional, Tuple, Type, Union
+
+    from wandb.proto.wandb_internal_pb2 import Record, Result
 
     ExceptionType = Union[
         Tuple[Type[BaseException], BaseException, TracebackType],
