@@ -4,7 +4,7 @@ progress.
 
 import os
 import sys
-from typing import IO, Optional, TYPE_CHECKING
+from typing import IO, TYPE_CHECKING, Optional
 
 from wandb.errors import CommError
 
