@@ -3,38 +3,35 @@ sender.
 """
 
 
-from collections import defaultdict
-from datetime import datetime
 import json
 import logging
 import os
 import queue
-from queue import Queue
 import threading
 import time
+from collections import defaultdict
+from datetime import datetime
+from queue import Queue
 from typing import (
+    TYPE_CHECKING,
     Any,
-    cast,
     Dict,
     Generator,
     List,
     NewType,
     Optional,
     Tuple,
-    TYPE_CHECKING,
+    cast,
 )
 
-from pkg_resources import parse_version
 import requests
+
 import wandb
 from wandb import util
 from wandb.filesync.dir_watcher import DirWatcher
 from wandb.proto import wandb_internal_pb2
 from wandb.sdk.lib import redirect
 
-from . import artifacts, file_stream, internal_api, update
-from .file_pusher import FilePusher
-from .settings_static import SettingsDict, SettingsStatic
 from ..interface import interface
 from ..interface.interface_queue import InterfaceQueue
 from ..lib import (
@@ -48,9 +45,13 @@ from ..lib import (
 )
 from ..lib.proto_util import message_to_dict
 from ..wandb_settings import Settings
-
+from . import artifacts, file_stream, internal_api, update
+from .file_pusher import FilePusher
+from .settings_static import SettingsDict, SettingsStatic
 
 if TYPE_CHECKING:
+    import sys
+
     from wandb.proto.wandb_internal_pb2 import (
         ArtifactRecord,
         HttpResponse,
@@ -61,8 +62,6 @@ if TYPE_CHECKING:
         RunRecord,
     )
 
-    import sys
-
     if sys.version_info >= (3, 8):
         from typing import Literal
     else:
@@ -170,6 +169,7 @@ class SendManager:
     _project: "Optional[str]"
     _dir_watcher: "Optional[DirWatcher]"
     _pusher: "Optional[FilePusher]"
+    _record_exit: "Optional[Record]"
     _exit_result: "Optional[RunExitResult]"
     _resume_state: ResumeState
     _cached_server_info: Dict[str, Any]
@@ -220,7 +220,8 @@ class SendManager:
         # State updated by resuming
         self._resume_state = ResumeState()
 
-        # State added when run_exit is complete
+        # State added when run_exit is initiated and complete
+        self._record_exit = None
         self._exit_result = None
 
         self._api = internal_api.Api(
@@ -433,11 +434,14 @@ class SendManager:
             self._respond_result(result)
 
     def send_exit(self, record: "Record") -> None:
-        exit = record.exit
-        self._exit_code = exit.exit_code
-        logger.info("handling exit code: %s", exit.exit_code)
-        runtime = exit.runtime
-        logger.info("handling runtime: %s", exit.runtime)
+        # track where the exit came from
+        self._record_exit = record
+
+        run_exit = record.exit
+        self._exit_code = run_exit.exit_code
+        logger.info("handling exit code: %s", run_exit.exit_code)
+        runtime = run_exit.runtime
+        logger.info("handling runtime: %s", run_exit.runtime)
         self._metadata_summary["runtime"] = runtime
         self._update_summary()
 
@@ -449,7 +453,7 @@ class SendManager:
     def send_final(self, record: "Record") -> None:
         pass
 
-    def send_request_defer(self, record: "Record") -> None:
+    def send_request_defer(self, record: "Record") -> None:  # noqa: C901
         defer = record.request.defer
         state = defer.state
         logger.info(f"handle sender defer: {state}")
@@ -494,6 +498,10 @@ class SendManager:
                 self._pusher.finish(transition_state)
             else:
                 transition_state()
+        elif state == defer.JOIN_FP:
+            if self._pusher:
+                self._pusher.join()
+            transition_state()
         elif state == defer.FLUSH_FS:
             if self._fs:
                 # TODO(jhr): now is a good time to output pending output lines
@@ -517,15 +525,20 @@ class SendManager:
         # mark exit done in case we are polling on exit
         self._exit_result = exit_result
 
+        # Report response to mailbox
+        if self._record_exit and self._record_exit.control.mailbox_slot:
+            result = proto_util._result_from_record(self._record_exit)
+            result.exit_result.CopyFrom(exit_result)
+            self._respond_result(result)
+
     def send_request_poll_exit(self, record: "Record") -> None:
-        if not record.control.req_resp:
+        if not record.control.req_resp and not record.control.mailbox_slot:
             return
 
         result = proto_util._result_from_record(record)
 
-        alive = False
         if self._pusher:
-            alive, status = self._pusher.get_status()
+            _alive, status = self._pusher.get_status()
             file_counts = self._pusher.file_counts_by_category()
             resp = result.response.poll_exit_response
             resp.pusher_stats.uploaded_bytes = status.uploaded_bytes
@@ -536,30 +549,32 @@ class SendManager:
             resp.file_counts.artifact_count = file_counts.artifact
             resp.file_counts.other_count = file_counts.other
 
-        if self._exit_result and not alive:
-            # pusher join should not block as it was reported as not alive
-            if self._pusher:
-                self._pusher.join()
-            result.response.poll_exit_response.exit_result.CopyFrom(self._exit_result)
-            result.response.poll_exit_response.local_info.CopyFrom(
-                self.get_local_info()
-            )
+        if self._exit_result:
             result.response.poll_exit_response.done = True
-            for message in self._server_messages:
-                # guard agains the case the message level returns malformed from server
-                message_level = str(message.get("messageLevel"))
-                message_level_sanitized = int(
-                    printer.INFO if not message_level.isdigit() else message_level
-                )
-                result.response.poll_exit_response.server_messages.item.append(
-                    wandb_internal_pb2.ServerMessage(
-                        utf_text=message.get("utfText", ""),
-                        plain_text=message.get("plainText", ""),
-                        html_text=message.get("htmlText", ""),
-                        type=message.get("messageType", ""),
-                        level=message_level_sanitized,
-                    )
+            result.response.poll_exit_response.exit_result.CopyFrom(self._exit_result)
+
+        self._respond_result(result)
+
+    def send_request_server_info(self, record: "Record") -> None:
+        assert record.control.req_resp or record.control.mailbox_slot
+        result = proto_util._result_from_record(record)
+
+        result.response.server_info_response.local_info.CopyFrom(self.get_local_info())
+        for message in self._server_messages:
+            # guard agains the case the message level returns malformed from server
+            message_level = str(message.get("messageLevel"))
+            message_level_sanitized = int(
+                printer.INFO if not message_level.isdigit() else message_level
+            )
+            result.response.server_info_response.server_messages.item.append(
+                wandb_internal_pb2.ServerMessage(
+                    utf_text=message.get("utfText", ""),
+                    plain_text=message.get("plainText", ""),
+                    html_text=message.get("htmlText", ""),
+                    type=message.get("messageType", ""),
+                    level=message_level_sanitized,
                 )
+            )
         self._respond_result(result)
 
     def _maybe_setup_resume(
@@ -752,7 +767,7 @@ class SendManager:
             error = self._maybe_setup_resume(run)
 
         if error is not None:
-            if record.control.req_resp:
+            if record.control.req_resp or record.control.mailbox_slot:
                 result = proto_util._result_from_record(record)
                 result.run_result.run.CopyFrom(run)
                 result.run_result.error.CopyFrom(error)
@@ -782,7 +797,7 @@ class SendManager:
         self._init_run(run, config_value_dict)
         assert self._run  # self._run is configured in _init_run()
 
-        if record.control.req_resp:
+        if record.control.req_resp or record.control.mailbox_slot:
             result = proto_util._result_from_record(record)
             # TODO: we could do self._interface.publish_defer(resp) to notify
             # the handler not to actually perform server updates for this uuid
@@ -1251,6 +1266,8 @@ class SendManager:
     def _send_artifact(
         self, artifact: "ArtifactRecord", history_step: Optional[int] = None
     ) -> Optional[Dict]:
+        from pkg_resources import parse_version
+
         assert self._pusher
         saver = artifacts.ArtifactSaver(
             api=self._api,
@@ -1288,6 +1305,8 @@ class SendManager:
         )
 
     def send_alert(self, record: "Record") -> None:
+        from pkg_resources import parse_version
+
         alert = record.alert
         max_cli_version = self._max_cli_version()
         if max_cli_version is None or parse_version(max_cli_version) < parse_version(
