@@ -7,19 +7,19 @@ import enum
 import json
 import logging
 import os
-from shlex import quote
 import tempfile
+from shlex import quote
 from typing import Any, Dict, List, Optional
 
 import wandb
+import wandb.docker as docker
 from wandb.apis.internal import Api
 from wandb.apis.public import Artifact as PublicArtifact
-import wandb.docker as docker
 from wandb.errors import CommError, LaunchError
 from wandb.sdk.lib.runid import generate_id
 
 from . import utils
-
+from .utils import LOG_PREFIX
 
 _logger = logging.getLogger(__name__)
 
@@ -61,9 +61,10 @@ class LaunchProject:
     ):
         if uri is not None and utils.is_bare_wandb_uri(uri):
             uri = api.settings("base_url") + uri
-            _logger.info(f"Updating uri with base uri: {uri}")
+            _logger.info(f"{LOG_PREFIX}Updating uri with base uri: {uri}")
         self.uri = uri
         self.job = job
+        wandb.termlog(f"{LOG_PREFIX}Launch project got job {job}")
         self._job_artifact: Optional[PublicArtifact] = None
         self.api = api
         self.launch_spec = launch_spec
@@ -80,7 +81,7 @@ class LaunchProject:
         uid = RESOURCE_UID_MAP.get(resource, 1000)
         if self._base_image:
             uid = docker.get_image_uid(self._base_image)
-            _logger.info(f"Retrieved base image uid {uid}")
+            _logger.info(f"{LOG_PREFIX}Retrieved base image uid {uid}")
         self.docker_user_id: int = docker_config.get("user_id", uid)
         self.git_version: Optional[str] = git_info.get("version")
         self.git_repo: Optional[str] = git_info.get("repo")
@@ -93,6 +94,7 @@ class LaunchProject:
         self._runtime: Optional[str] = None
         self.run_id = run_id or generate_id()
         self._image_tag: str = self._initialize_image_job_tag() or self.run_id
+        wandb.termlog(f"{LOG_PREFIX}Launch project using image tag {self._image_tag}")
         self._entry_points: Dict[
             str, EntryPoint
         ] = {}  # todo: keep multiple entrypoint support?
@@ -116,6 +118,13 @@ class LaunchProject:
             _logger.info(f"URI {self.uri} indicates a git uri")
             self.source = LaunchSource.GIT
             self.project_dir = tempfile.mkdtemp()
+        elif self.uri is not None and "placeholder-" in self.uri:
+            wandb.termlog(
+                f"{LOG_PREFIX}Launch received placeholder URI, replacing with local path."
+            )
+            self.uri = os.getcwd()
+            self.source = LaunchSource.LOCAL
+            self.project_dir = self.uri
         else:
             _logger.info(f"URI {self.uri} indicates a local uri")
             # assume local
@@ -162,9 +171,11 @@ class LaunchProject:
 
     def _initialize_image_job_tag(self) -> Optional[str]:
         if self.job is not None:
-            _, alias = self.job.split(":")
-            self._image_tag = alias
-            return alias
+            job_name, alias = self.job.split(":")
+            # Alias is used to differentiate images between jobs of the same sequence
+            _image_tag = f"{alias}-{job_name}"
+            _logger.debug(f"{LOG_PREFIX}Setting image tag {_image_tag}")
+            return wandb.util.make_docker_image_name_safe(_image_tag)
         return None
 
     @property
@@ -175,6 +186,7 @@ class LaunchProject:
 
     @property
     def image_tag(self) -> str:
+
         return self._image_tag[:IMAGE_TAG_MAX_LENGTH]
 
     @property
@@ -252,9 +264,7 @@ class LaunchProject:
                 if self.cuda is None:
                     # only set cuda on by default if cuda is None (unspecified), not False (user specifically requested cpu image)
                     wandb.termlog(
-                        "Original wandb run {} was run with cuda version {}. Enabling cuda builds by default; to build on a CPU-only image, run again with --cuda=False".format(
-                            source_run_name, original_cuda_version
-                        )
+                        f"{LOG_PREFIX}Original wandb run {source_run_name} was run with cuda version {original_cuda_version}. Enabling cuda builds by default; to build on a CPU-only image, run again with --cuda=False"
                     )
                     self.cuda_version = original_cuda_version
                     self.cuda = True
@@ -264,9 +274,7 @@ class LaunchProject:
                     and self.cuda_version != original_cuda_version
                 ):
                     wandb.termlog(
-                        "Specified cuda version {} differs from original cuda version {}. Running with specified version {}".format(
-                            self.cuda_version, original_cuda_version, self.cuda_version
-                        )
+                        f"{LOG_PREFIX}Specified cuda version {self.cuda_version} differs from original cuda version {original_cuda_version}. Running with specified version {self.cuda_version}"
                     )
             # Specify the python runtime for jupyter2docker
             self.python_version = run_info.get("python", "3")
@@ -287,11 +295,13 @@ class LaunchProject:
                     raise LaunchError(
                         "Reproducing a run requires either an associated git repo or a code artifact logged with `run.log_code()`"
                     )
-                utils._fetch_git_repo(
+                branch_name = utils._fetch_git_repo(
                     self.project_dir,
                     run_info["git"]["remote"],
                     run_info["git"]["commit"],
                 )
+                if self.git_version is None:
+                    self.git_version = branch_name
                 patch = utils.fetch_project_diff(
                     source_entity, source_project, source_run_name, internal_api
                 )
@@ -354,10 +364,14 @@ class LaunchProject:
             )
             if not self._entry_points:
                 wandb.termlog(
-                    "Entry point for repo not specified, defaulting to python main.py"
+                    f"{LOG_PREFIX}Entry point for repo not specified, defaulting to python main.py"
                 )
                 self.add_entry_point(["python", "main.py"])
-            utils._fetch_git_repo(self.project_dir, self.uri, self.git_version)
+            branch_name = utils._fetch_git_repo(
+                self.project_dir, self.uri, self.git_version
+            )
+            if self.git_version is None:
+                self.git_version = branch_name
 
 
 class EntryPoint:
@@ -456,7 +470,7 @@ def fetch_and_validate_project(
     if launch_project.source == LaunchSource.LOCAL:
         if not launch_project._entry_points:
             wandb.termlog(
-                "Entry point for repo not specified, defaulting to `python main.py`"
+                f"{LOG_PREFIX}Entry point for repo not specified, defaulting to `python main.py`"
             )
             launch_project.add_entry_point(["python", "main.py"])
     elif launch_project.source == LaunchSource.JOB:
