@@ -6,18 +6,19 @@ import logging
 import os
 import pprint
 import time
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, Dict, List, Union
 
 import wandb
+import wandb.util as util
 from wandb.apis.internal import Api
 from wandb.sdk.launch.runner.local_container import LocalSubmittedRun
-import wandb.util as util
 
 from .._project_spec import create_project_from_spec, fetch_and_validate_project
 from ..builder.loader import load_builder
 from ..runner.abstract import AbstractRun
 from ..runner.loader import load_backend
 from ..utils import (
+    LOG_PREFIX,
     PROJECT_DOCKER_ARGS,
     PROJECT_SYNCHRONOUS,
     resolve_build_and_registry_config,
@@ -55,11 +56,12 @@ class LaunchAgent:
         self._cwd = os.getcwd()
         self._namespace = wandb.util.generate_id()
         self._access = _convert_access("project")
-        if config.get("max_jobs") == -1:
+        max_jobs_from_config = int(config.get("max_jobs", 1))
+        if max_jobs_from_config == -1:
             self._max_jobs = float("inf")
         else:
-            self._max_jobs = config.get("max_jobs") or 1
-        self.default_config: Optional[Dict[str, Any]] = config
+            self._max_jobs = max_jobs_from_config
+        self.default_config: Dict[str, Any] = config
 
         # serverside creation
         self.gorilla_supports_agents = (
@@ -97,9 +99,7 @@ class LaunchAgent:
     def print_status(self) -> None:
         """Prints the current status of the agent."""
         wandb.termlog(
-            "agent {} polling on project {}, queues {} for jobs".format(
-                self._name, self._project, " ".join(self._queues)
-            )
+            f"{LOG_PREFIX}agent {self._name} polling on project {self._project}, queues {','.join(self._queues)} for jobs"
         )
 
     def update_status(self, status: str) -> None:
@@ -145,7 +145,7 @@ class LaunchAgent:
 
     def run_job(self, job: Dict[str, Any]) -> None:
         """Sets up project and runs the job."""
-        _msg = f"Launch agent received job:\n{pprint.pformat(job)}\n"
+        _msg = f"{LOG_PREFIX}Launch agent received job:\n{pprint.pformat(job)}\n"
         wandb.termlog(_msg)
         _logger.info(_msg)
         # update agent status
@@ -168,7 +168,8 @@ class LaunchAgent:
         _logger.info("Fetching resource...")
         resource = launch_spec.get("resource") or "local-container"
         backend_config: Dict[str, Any] = {
-            PROJECT_DOCKER_ARGS: {},
+            PROJECT_DOCKER_ARGS: (launch_spec.get("docker", {}) or {}).get("args", {})
+            or {},
             PROJECT_SYNCHRONOUS: False,  # agent always runs async
         }
 
@@ -181,6 +182,10 @@ class LaunchAgent:
             self.default_config, override_build_config, override_registry_config
         )
         builder = load_builder(build_config)
+
+        default_runner = self.default_config.get("runner", {}).get("type")
+        if default_runner == resource:
+            backend_config["runner"] = self.default_config.get("runner")
         backend = load_backend(resource, self._api, backend_config)
         backend.verify()
         _logger.info("Backend loaded...")
@@ -191,11 +196,7 @@ class LaunchAgent:
 
     def loop(self) -> None:
         """Main loop function for agent."""
-        wandb.termlog(
-            "launch agent polling project {}/{} on queues: {}".format(
-                self._entity, self._project, ",".join(self._queues)
-            )
-        )
+        self.print_status()
         try:
             while True:
                 self._ticks += 1
@@ -237,5 +238,5 @@ class LaunchAgent:
                 if isinstance(run, LocalSubmittedRun):
                     run.command_proc.kill()
             self.update_status(AGENT_KILLED)
-            wandb.termlog("Shutting down, active jobs:")
+            wandb.termlog(f"{LOG_PREFIX}Shutting down, active jobs:")
             self.print_status()
