@@ -1,15 +1,17 @@
-from abc import ABC, abstractmethod
 import logging
 import os
 import subprocess
 import sys
+from abc import ABC, abstractmethod
 from typing import Any, Dict, List, Optional, Union
 
 from dockerpycreds.utils import find_executable  # type: ignore
+
 import wandb
 from wandb import Settings
 from wandb.apis.internal import Api
 from wandb.errors import CommError
+from wandb.sdk.launch.builder.abstract import AbstractBuilder
 
 from .._project_spec import LaunchProject
 
@@ -154,7 +156,12 @@ class AbstractRunner(ABC):
         return True
 
     @abstractmethod
-    def run(self, launch_project: LaunchProject) -> Optional[AbstractRun]:
+    def run(
+        self,
+        launch_project: LaunchProject,
+        builder: AbstractBuilder,
+        registry_config: Dict[str, Any],
+    ) -> Optional[AbstractRun]:
         """Submit an LaunchProject to be run.
 
         Returns a SubmittedRun object to track the execution
