@@ -1,16 +1,47 @@
 # Note: this is a helper printer class, this file might go away once we switch to rich console printing
 
-from abc import abstractmethod
 import itertools
 import platform
 import sys
-from typing import List, Optional, Tuple, Union
+from abc import abstractmethod
+from typing import Callable, List, Optional, Tuple, Union
 
 import click
+
 import wandb
 
 from . import ipython, sparkline
 
+# Follow the same logic as the python logging module
+CRITICAL = 50
+FATAL = CRITICAL
+ERROR = 40
+WARNING = 30
+WARN = WARNING
+INFO = 20
+DEBUG = 10
+NOTSET = 0
+
+_level_to_name = {
+    CRITICAL: "CRITICAL",
+    ERROR: "ERROR",
+    WARNING: "WARNING",
+    INFO: "INFO",
+    DEBUG: "DEBUG",
+    NOTSET: "NOTSET",
+}
+
+_name_to_level = {
+    "CRITICAL": CRITICAL,
+    "FATAL": FATAL,
+    "ERROR": ERROR,
+    "WARN": WARNING,
+    "WARNING": WARNING,
+    "INFO": INFO,
+    "DEBUG": DEBUG,
+    "NOTSET": NOTSET,
+}
+
 
 class _Printer:
     def sparklines(self, series: List[Union[int, float]]) -> Optional[str]:
@@ -19,25 +50,51 @@ class _Printer:
             return sparkline.sparkify(series)
         return None
 
-    def abort(self,) -> str:
+    def abort(
+        self,
+    ) -> str:
         return "Control-C" if platform.system() != "Windows" else "Ctrl-C"
 
     def display(
         self,
         text: Union[str, List[str], Tuple[str]],
         *,
-        status: Optional[str] = None,
+        level: Optional[Union[str, int]] = None,
         off: Optional[bool] = None,
+        default_text: Optional[Union[str, List[str], Tuple[str]]] = None,
     ) -> None:
-        if not off:
-            self._display(text, status=status)
+        if off:
+            return
+        self._display(text, level=level, default_text=default_text)
 
     @abstractmethod
     def _display(
-        self, text: Union[str, List[str], Tuple[str]], *, status: Optional[str] = None,
+        self,
+        text: Union[str, List[str], Tuple[str]],
+        *,
+        level: Optional[Union[str, int]] = None,
+        default_text: Optional[Union[str, List[str], Tuple[str]]] = None,
     ) -> None:
         raise NotImplementedError
 
+    @staticmethod
+    def _sanitize_level(name_or_level: Optional[Union[str, int]]) -> int:
+        if isinstance(name_or_level, str):
+            try:
+                return _name_to_level[name_or_level.upper()]
+            except KeyError:
+                raise ValueError(
+                    f"Unknown level name: {name_or_level}, supported levels: {_name_to_level.keys()}"
+                )
+
+        if isinstance(name_or_level, int):
+            return name_or_level
+
+        if name_or_level is None:
+            return INFO
+
+        raise ValueError(f"Unknown status level {name_or_level}")
+
     @abstractmethod
     def code(self, text: str) -> str:
         raise NotImplementedError
@@ -78,32 +135,56 @@ class PrinterTerm(_Printer):
         self._progress = itertools.cycle(["-", "\\", "|", "/"])
 
     def _display(
-        self, text: Union[str, List[str], Tuple[str]], *, status: Optional[str] = None
+        self,
+        text: Union[str, List[str], Tuple[str]],
+        *,
+        level: Optional[Union[str, int]] = None,
+        default_text: Optional[Union[str, List[str], Tuple[str]]] = None,
     ) -> None:
         text = "\n".join(text) if isinstance(text, (list, tuple)) else text
-        if status == "info" or status is None:
-            wandb.termlog(text)
-        elif status == "warn":
-            wandb.termwarn(text)
-        elif status == "error":
-            wandb.termerror(text)
-        else:
-            raise
+        if default_text is not None:
+            default_text = (
+                "\n".join(default_text)
+                if isinstance(default_text, (list, tuple))
+                else default_text
+            )
+            text = text or default_text
+        self._display_fn_mapping(level)(text)
 
-    def progress_update(self, text: str, percentage: Optional[float] = None) -> None:
+    @staticmethod
+    def _display_fn_mapping(level: Optional[Union[str, int]]) -> Callable[[str], None]:
+        level = _Printer._sanitize_level(level)
+
+        if level >= CRITICAL:
+            return wandb.termerror
+        elif ERROR <= level < CRITICAL:
+            return wandb.termerror
+        elif WARNING <= level < ERROR:
+            return wandb.termwarn
+        elif INFO <= level < WARNING:
+            return wandb.termlog
+        elif DEBUG <= level < INFO:
+            return wandb.termlog
+        else:
+            return wandb.termlog
+
+    def progress_update(self, text: str, percent_done: Optional[float] = None) -> None:
         wandb.termlog(f"{next(self._progress)} {text}", newline=False)
 
     def progress_close(self) -> None:
         wandb.termlog(" " * 79)
 
     def code(self, text: str) -> str:
-        return click.style(text, bold=True)
+        ret: str = click.style(text, bold=True)
+        return ret
 
     def name(self, text: str) -> str:
-        return click.style(text, fg="yellow")
+        ret: str = click.style(text, fg="yellow")
+        return ret
 
     def link(self, link: str, text: Optional[str] = None) -> str:
-        return click.style(link, fg="blue", underline=True)
+        ret: str = click.style(link, fg="blue", underline=True)
+        return ret
 
     def emoji(self, name: str) -> str:
         emojis = dict()
@@ -114,13 +195,15 @@ class PrinterTerm(_Printer):
 
     def status(self, text: str, failure: Optional[bool] = None) -> str:
         color = "red" if failure else "green"
-        return click.style(text, fg=color)
+        ret: str = click.style(text, fg=color)
+        return ret
 
     def files(self, text: str) -> str:
-        return click.style(text, fg="magenta", bold=True)
+        ret: str = click.style(text, fg="magenta", bold=True)
+        return ret
 
     def grid(self, rows: List[List[str]], title: Optional[str] = None) -> str:
-        max_len = max([len(row[0]) for row in rows])
+        max_len = max(len(row[0]) for row in rows)
         format_row = " ".join(["{:>{max_len}}", "{}" * (len(rows[0]) - 1)])
         grid = "\n".join([format_row.format(*row, max_len=max_len) for row in rows])
         if title:
@@ -138,17 +221,38 @@ class PrinterJupyter(_Printer):
         self._progress = ipython.jupyter_progress_bar()
 
     def _display(
-        self, text: Union[str, List[str], Tuple[str]], *, status: Optional[str] = None
+        self,
+        text: Union[str, List[str], Tuple[str]],
+        *,
+        level: Optional[Union[str, int]] = None,
+        default_text: Optional[Union[str, List[str], Tuple[str]]] = None,
     ) -> None:
         text = "<br/>".join(text) if isinstance(text, (list, tuple)) else text
-        if status == "info" or status is None:
-            ipython.display_html(text)
-        elif status == "warn":
-            ipython.display_html(text)
-        elif status == "error":
-            ipython.display_html(text)
+        if default_text is not None:
+            default_text = (
+                "<br/>".join(default_text)
+                if isinstance(default_text, (list, tuple))
+                else default_text
+            )
+            text = text or default_text
+        self._display_fn_mapping(level)(text)
+
+    @staticmethod
+    def _display_fn_mapping(level: Optional[Union[str, int]]) -> Callable[[str], None]:
+        level = _Printer._sanitize_level(level)
+
+        if level >= CRITICAL:
+            return ipython.display_html
+        elif ERROR <= level < CRITICAL:
+            return ipython.display_html
+        elif WARNING <= level < ERROR:
+            return ipython.display_html
+        elif INFO <= level < WARNING:
+            return ipython.display_html
+        elif DEBUG <= level < INFO:
+            return ipython.display_html
         else:
-            raise
+            return ipython.display_html
 
     def code(self, text: str) -> str:
         return f"<code>{text}<code>"
