@@ -1,18 +1,19 @@
 #
-from datetime import datetime
 import json
-from typing import Any, Dict, Union
-from typing import TYPE_CHECKING
+from datetime import datetime
+from typing import TYPE_CHECKING, Any, Dict, Union
 
 from wandb.proto import wandb_internal_pb2 as pb
 
-
 if TYPE_CHECKING:  # pragma: no cover
+    from google.protobuf.internal.containers import (
+        MessageMap,
+        RepeatedCompositeFieldContainer,
+    )
+    from google.protobuf.message import Message
+
     from wandb.proto import wandb_server_pb2 as spb
     from wandb.proto import wandb_telemetry_pb2 as tpb
-    from google.protobuf.internal.containers import MessageMap
-    from google.protobuf.internal.containers import RepeatedCompositeFieldContainer
-    from google.protobuf.message import Message
 
 
 def dict_from_proto_list(obj_list: "RepeatedCompositeFieldContainer") -> Dict[str, Any]:
@@ -82,7 +83,9 @@ def settings_dict_from_pbmap(
     return d
 
 
-def message_to_dict(message: "Message",) -> Dict[str, Any]:
+def message_to_dict(
+    message: "Message",
+) -> Dict[str, Any]:
     """
     Converts a protobuf message into a dictionary.
     """
