@@ -2,8 +2,8 @@ import socket
 import struct
 import threading
 import time
-from typing import Any, Optional, TYPE_CHECKING
 import uuid
+from typing import TYPE_CHECKING, Any, List, Optional
 
 from wandb.proto import wandb_server_pb2 as spb
 
@@ -19,27 +19,94 @@ class SockClientClosedError(Exception):
     pass
 
 
+class SockBuffer:
+    _buf_list: List[bytes]
+    _buf_lengths: List[int]
+    _buf_total: int
+
+    def __init__(self) -> None:
+        self._buf_list = []
+        self._buf_lengths = []
+        self._buf_total = 0
+
+    @property
+    def length(self) -> int:
+        return self._buf_total
+
+    def _get(self, start: int, end: int, peek: bool = False) -> bytes:
+        index: Optional[int] = None
+        buffers = []
+        need = end
+
+        # compute buffers needed
+        for i, (buf_len, buf_data) in enumerate(zip(self._buf_lengths, self._buf_list)):
+            buffers.append(buf_data[:need] if need < buf_len else buf_data)
+            if need <= buf_len:
+                index = i
+                break
+            need -= buf_len
+
+        # buffer not large enough, caller should have made sure there was enough data
+        if index is None:
+            raise IndexError("SockBuffer index out of range")
+
+        # advance buffer internals if we are not peeking into the data
+        if not peek:
+            self._buf_total -= end
+            if need < buf_len:
+                # update partially used buffer list
+                self._buf_list = self._buf_list[index:]
+                self._buf_lengths = self._buf_lengths[index:]
+                self._buf_list[0] = self._buf_list[0][need:]
+                self._buf_lengths[0] -= need
+            else:
+                # update fully used buffer list
+                self._buf_list = self._buf_list[index + 1 :]
+                self._buf_lengths = self._buf_lengths[index + 1 :]
+
+        return b"".join(buffers)[start:end]
+
+    def get(self, start: int, end: int) -> bytes:
+        return self._get(start, end)
+
+    def peek(self, start: int, end: int) -> bytes:
+        return self._get(start, end, peek=True)
+
+    def put(self, data: bytes, data_len: int) -> None:
+        self._buf_list.append(data)
+        self._buf_lengths.append(data_len)
+        self._buf_total += data_len
+
+
 class SockClient:
     _sock: socket.socket
-    _data: bytes
     _sockid: str
     _retry_delay: float
     _lock: "threading.Lock"
+    _bufsize: int
+    _buffer: SockBuffer
 
     # current header is magic byte "W" followed by 4 byte length of the message
     HEADLEN = 1 + 4
 
     def __init__(self) -> None:
-        self._data = b""
         # TODO: use safe uuid's (python3.7+) or emulate this
         self._sockid = uuid.uuid4().hex
         self._retry_delay = 0.1
         self._lock = threading.Lock()
+        self._bufsize = 4096
+        self._buffer = SockBuffer()
 
     def connect(self, port: int) -> None:
         s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         s.connect(("localhost", port))
         self._sock = s
+        self._detect_bufsize()
+
+    def _detect_bufsize(self) -> None:
+        sndbuf_size = self._sock.getsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF)
+        rcvbuf_size = self._sock.getsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF)
+        self._bufsize = min(sndbuf_size, rcvbuf_size, 65536)
 
     def close(self) -> None:
         self._sock.close()
@@ -49,20 +116,24 @@ class SockClient:
 
     def set_socket(self, sock: socket.socket) -> None:
         self._sock = sock
+        self._detect_bufsize()
 
     def _sendall_with_error_handle(self, data: bytes) -> None:
         # This is a helper function for sending data in a retry fashion.
         # Similar to the sendall() function in the socket module, but with a
         # an error handling in case of timeout.
         total_sent = 0
-        while total_sent < len(data):
+        total_data = len(data)
+        while total_sent < total_data:
             start_time = time.monotonic()
             try:
-                sent = self._sock.send(data[total_sent:])
+                sent = self._sock.send(data)
                 # sent equal to 0 indicates a closed socket
                 if sent == 0:
                     raise SockClientClosedError("socket connection broken")
                 total_sent += sent
+                # truncate our data to save memory
+                data = data[sent:]
             # we handle the timeout case for the cases when timeout is set
             # on a system level by another application
             except socket.timeout:
@@ -151,18 +222,16 @@ class SockClient:
 
     def _extract_packet_bytes(self) -> Optional[bytes]:
         # Do we have enough data to read the header?
-        len_data = len(self._data)
         start_offset = self.HEADLEN
-        if len_data >= start_offset:
-            header = self._data[:start_offset]
+        if self._buffer.length >= start_offset:
+            header = self._buffer.peek(0, start_offset)
             fields = struct.unpack("<BI", header)
             magic, dlength = fields
             assert magic == ord("W")
             # Do we have enough data to read the full record?
             end_offset = self.HEADLEN + dlength
-            if len_data >= end_offset:
-                rec_data = self._data[start_offset:end_offset]
-                self._data = self._data[end_offset:]
+            if self._buffer.length >= end_offset:
+                rec_data = self._buffer.get(start_offset, end_offset)
                 return rec_data
         return None
 
@@ -183,7 +252,7 @@ class SockClient:
             if timeout:
                 self._sock.settimeout(timeout)
             try:
-                data = self._sock.recv(4096)
+                data = self._sock.recv(self._bufsize)
             except socket.timeout:
                 break
             except ConnectionResetError:
@@ -193,11 +262,12 @@ class SockClient:
             finally:
                 if timeout:
                     self._sock.settimeout(None)
-            if len(data) == 0:
+            data_len = len(data)
+            if data_len == 0:
                 # socket.recv() will return 0 bytes if socket was shutdown
                 # caller will handle this condition like other connection problems
                 raise SockClientClosedError()
-            self._data += data
+            self._buffer.put(data, data_len)
         return None
 
     def read_server_request(self) -> Optional[spb.ServerRequest]:
