@@ -2,20 +2,19 @@ import queue
 import socket
 import threading
 import time
-from typing import Any, Callable, Dict, Optional
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Any, Callable, Dict, Optional
 
 from wandb.proto import wandb_server_pb2 as spb
 
-from .service_base import _pbmap_apply_dict
-from .streams import StreamMux
 from ..lib import tracelog
 from ..lib.proto_util import settings_dict_from_pbmap
 from ..lib.sock_client import SockClient, SockClientClosedError
-
+from .service_base import _pbmap_apply_dict
+from .streams import StreamMux
 
 if TYPE_CHECKING:
     from threading import Event
+
     from ..interface.interface_relay import InterfaceRelay
 
 
@@ -105,8 +104,8 @@ class SockServerReadThread(threading.Thread):
                 break
             assert sreq, "read_server_request should never timeout"
             sreq_type = sreq.WhichOneof("server_request_type")
-            shandler_str = "server_" + sreq_type
-            shandler: "Callable[[spb.ServerRequest], None]" = getattr(
+            shandler_str = "server_" + sreq_type  # type: ignore
+            shandler: "Callable[[spb.ServerRequest], None]" = getattr(  # type: ignore
                 self, shandler_str, None
             )
             assert shandler, f"unknown handle: {shandler_str}"
@@ -168,6 +167,8 @@ class SockServerReadThread(threading.Thread):
 
     def server_record_publish(self, sreq: "spb.ServerRequest") -> None:
         record = sreq.record_publish
+        # encode relay information so the right socket picks up the data
+        record.control.relay_id = self._sock_client._sockid
         stream_id = record._info.stream_id
         iface = self._mux.get_stream(stream_id).interface
         assert iface.record_q
