@@ -3,15 +3,14 @@
 Implement ServiceInterface for grpc transport.
 """
 
-from typing import Optional
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Optional
 
 import grpc
+
 from wandb.proto import wandb_server_pb2 as spb
 from wandb.proto import wandb_server_pb2_grpc as pbgrpc
 
-from .service_base import _pbmap_apply_dict
-from .service_base import ServiceInterface
+from .service_base import ServiceInterface, _pbmap_apply_dict
 
 if TYPE_CHECKING:
     from wandb.sdk.wandb_settings import Settings
@@ -27,7 +26,7 @@ class ServiceGrpcInterface(ServiceInterface):
         return "grpc"
 
     def _svc_connect(self, port: int) -> None:
-        channel = grpc.insecure_channel("localhost:{}".format(port))
+        channel = grpc.insecure_channel(f"localhost:{port}")
         stub = pbgrpc.InternalServiceStub(channel)
         self._stub = stub
         # TODO: make sure service is up
@@ -67,7 +66,7 @@ class ServiceGrpcInterface(ServiceInterface):
 
         inform_attach = spb.ServerInformAttachRequest()
         inform_attach._info.stream_id = attach_id
-        return self._stub.ServerInformAttach(inform_attach)
+        return self._stub.ServerInformAttach(inform_attach)  # type: ignore
 
     def _svc_inform_teardown(self, exit_code: int) -> None:
         inform_teardown = spb.ServerInformTeardownRequest(exit_code=exit_code)
