@@ -1,6 +1,7 @@
 import base64
 import contextlib
 import hashlib
+import json
 import os
 import pathlib
 import re
@@ -8,55 +9,68 @@ import shutil
 import tempfile
 import time
 from typing import (
+    IO,
+    TYPE_CHECKING,
     Any,
-    Callable,
     Dict,
     Generator,
-    IO,
     List,
     Mapping,
     Optional,
     Sequence,
     Tuple,
-    TYPE_CHECKING,
     Union,
+    cast,
 )
 from urllib.parse import parse_qsl, quote, urlparse
 
 import requests
+import urllib3
+
 import wandb
-from wandb import env
-from wandb import util
+import wandb.data_types as data_types
+from wandb import env, util
 from wandb.apis import InternalApi, PublicApi
 from wandb.apis.public import Artifact as PublicArtifact
-import wandb.data_types as data_types
 from wandb.errors import CommError
 from wandb.errors.term import termlog, termwarn
+from wandb.sdk.internal import progress
 
 from . import lib as wandb_lib
-from .interface.artifacts import (  # noqa: F401 pylint: disable=unused-import
-    Artifact as ArtifactInterface,
+from .data_types._dtypes import Type, TypeRegistry
+from .interface.artifacts import Artifact as ArtifactInterface
+from .interface.artifacts import (  # noqa: F401
     ArtifactEntry,
     ArtifactManifest,
     ArtifactsCache,
+    StorageHandler,
+    StorageLayout,
+    StoragePolicy,
     b64_string_to_hex,
     get_artifacts_cache,
     md5_file_b64,
     md5_string,
-    StorageHandler,
-    StorageLayout,
-    StoragePolicy,
 )
 
-
 if TYPE_CHECKING:
-    import google.cloud.storage as gcs_module  # type: ignore
+
+    # We could probably use https://pypi.org/project/boto3-stubs/ or something
+    # instead of `type:ignore`ing these boto imports, but it's nontrivial:
+    # for some reason, despite being actively maintained as of 2022-09-30,
+    # the latest release of boto3-stubs doesn't include all the features we use.
     import boto3  # type: ignore
-    import wandb.filesync.step_prepare.StepPrepare as StepPrepare  # type: ignore
+    import boto3.resources.base  # type: ignore
+    import boto3.s3  # type: ignore
+    import boto3.session  # type: ignore
+    import google.cloud.storage as gcs_module  # type: ignore
+
+    import wandb.apis.public
+    from wandb.filesync.step_prepare import StepPrepare
+    from wandb.sdk.internal import internal_api
 
 # This makes the first sleep 1s, and then doubles it up to total times,
 # which makes for ~18 hours.
-_REQUEST_RETRY_STRATEGY = requests.packages.urllib3.util.retry.Retry(
+_REQUEST_RETRY_STRATEGY = urllib3.util.retry.Retry(
     backoff_factor=1,
     total=16,
     status_forcelist=(308, 408, 409, 429, 500, 502, 503, 504),
@@ -75,6 +89,16 @@ class _AddedObj:
         self.obj = obj
 
 
+def _normalize_metadata(metadata: Optional[Dict[str, Any]]) -> Dict[str, Any]:
+    if metadata is None:
+        return {}
+    if not isinstance(metadata, dict):
+        raise TypeError(f"metadata must be dict, not {type(metadata)}")
+    return cast(
+        Dict[str, Any], json.loads(json.dumps(util.make_safe_for_json(metadata)))
+    )
+
+
 class Artifact(ArtifactInterface):
     """
     Flexible and lightweight building block for dataset and model versioning.
@@ -137,6 +161,7 @@ class Artifact(ArtifactInterface):
                 "Artifact name may only contain alphanumeric characters, dashes, underscores, and dots. "
                 'Invalid name: "%s"' % name
             )
+        metadata = _normalize_metadata(metadata)
         # TODO: this shouldn't be a property of the artifact. It's a more like an
         # argument to log_artifact.
         storage_layout = StorageLayout.V2
@@ -162,7 +187,7 @@ class Artifact(ArtifactInterface):
         self._type = type
         self._name = name
         self._description = description
-        self._metadata = metadata or {}
+        self._metadata = metadata
         self._distributed_id = None
         self._logged_artifact = None
         self._incremental = False
@@ -288,6 +313,7 @@ class Artifact(ArtifactInterface):
 
     @metadata.setter
     def metadata(self, metadata: dict) -> None:
+        metadata = _normalize_metadata(metadata)
         if self._logged_artifact:
             self._logged_artifact.metadata = metadata
             return
@@ -558,7 +584,7 @@ class Artifact(ArtifactInterface):
             "Cannot call get on an artifact before it has been logged or in offline mode"
         )
 
-    def download(self, root: str = None, recursive: bool = False) -> str:
+    def download(self, root: str = None, recursive: bool = False) -> util.FilePathStr:
         if self._logged_artifact:
             return self._logged_artifact.download(root=root, recursive=recursive)
 
@@ -631,9 +657,13 @@ class Artifact(ArtifactInterface):
             "Cannot call delete on an artifact before it has been logged or in offline mode"
         )
 
-    def wait(self) -> ArtifactInterface:
+    def wait(self, timeout: Optional[int] = None) -> ArtifactInterface:
+        """
+        Arguments:
+            timeout: (int, optional) Waits in seconds for artifact to finish logging if needed.
+        """
         if self._logged_artifact:
-            return self._logged_artifact.wait()
+            return self._logged_artifact.wait(timeout)  # type: ignore [call-arg]
 
         raise ValueError(
             "Cannot call wait on an artifact before it has been logged or in offline mode"
@@ -692,7 +722,7 @@ class Artifact(ArtifactInterface):
             raise ValueError("Can't add to finalized artifact.")
 
     def _add_local_file(
-        self, name: str, path: str, digest: Optional[str] = None
+        self, name: str, path: str, digest: Optional[util.B64MD5] = None
     ) -> ArtifactEntry:
         digest = digest or md5_file_b64(path)
         size = os.path.getsize(path)
@@ -763,7 +793,7 @@ class ArtifactManifestV1(ArtifactManifest):
     def __init__(
         self,
         artifact: ArtifactInterface,
-        storage_policy: StoragePolicy,
+        storage_policy: "WandbStoragePolicy",
         entries: Optional[Mapping[str, ArtifactEntry]] = None,
     ) -> None:
         super().__init__(artifact, storage_policy, entries=entries)
@@ -809,8 +839,8 @@ class ArtifactManifestEntry(ArtifactEntry):
     def __init__(
         self,
         path: str,
-        ref: Optional[str],
-        digest: str,
+        ref: Optional[Union[util.FilePathStr, util.URIStr]],
+        digest: Union[util.B64MD5, util.URIStr, util.FilePathStr, util.ETag],
         birth_artifact_id: Optional[str] = None,
         size: Optional[int] = None,
         extra: Dict = None,
@@ -894,7 +924,7 @@ class WandbStoragePolicy(StoragePolicy):
         self, artifact: ArtifactInterface, name: str, manifest_entry: ArtifactEntry
     ) -> str:
         path, hit, cache_open = self._cache.check_md5_obj_path(
-            manifest_entry.digest,
+            util.B64MD5(manifest_entry.digest),  # TODO(spencerpearson): unsafe cast
             manifest_entry.size if manifest_entry.size is not None else 0,
         )
         if hit:
@@ -965,25 +995,27 @@ class WandbStoragePolicy(StoragePolicy):
         artifact_manifest_id: str,
         entry: ArtifactEntry,
         preparer: "StepPrepare",
-        progress_callback: Optional[Callable] = None,
+        progress_callback: Optional["progress.ProgressFn"] = None,
     ) -> bool:
         # write-through cache
         cache_path, hit, cache_open = self._cache.check_md5_obj_path(
-            entry.digest, entry.size if entry.size is not None else 0
+            util.B64MD5(entry.digest),  # TODO(spencerpearson): unsafe cast
+            entry.size if entry.size is not None else 0,
         )
         if not hit and entry.local_path is not None:
             with cache_open() as f:
                 shutil.copyfile(entry.local_path, f.name)
             entry.local_path = cache_path
 
-        resp = preparer.prepare(
-            lambda: {
+        def _prepare_fn() -> "internal_api.CreateArtifactFileSpecInput":
+            return {
                 "artifactID": artifact_id,
                 "artifactManifestID": artifact_manifest_id,
                 "name": entry.path,
                 "md5": entry.digest,
             }
-        )
+
+        resp = preparer.prepare(_prepare_fn)
 
         entry.birth_artifact_id = resp.birth_artifact_id
         exists = resp.upload_url is None
@@ -1037,13 +1069,13 @@ class __S3BucketPolicy(StoragePolicy):
         artifact: ArtifactInterface,
         manifest_entry: ArtifactEntry,
         local: bool = False,
-    ) -> str:
+    ) -> Union[util.URIStr, util.FilePathStr]:
         return self._handler.load_path(artifact, manifest_entry, local=local)
 
     def store_path(
         self,
         artifact: Artifact,
-        path: str,
+        path: Union[util.URIStr, util.FilePathStr],
         name: Optional[str] = None,
         checksum: bool = True,
         max_objects: Optional[int] = None,
@@ -1077,7 +1109,7 @@ class MultiHandler(StorageHandler):
         artifact: ArtifactInterface,
         manifest_entry: ArtifactEntry,
         local: bool = False,
-    ) -> str:
+    ) -> Union[util.URIStr, util.FilePathStr]:
         url = urlparse(manifest_entry.ref)
         if url.scheme not in self._handlers:
             if self._default_handler is not None:
@@ -1140,7 +1172,7 @@ class TrackingHandler(StorageHandler):
         artifact: ArtifactInterface,
         manifest_entry: ArtifactEntry,
         local: bool = False,
-    ) -> str:
+    ) -> Union[util.URIStr, util.FilePathStr]:
         if local:
             # Likely a user error. The tracking handler is
             # oblivious to the underlying paths, so it has
@@ -1150,12 +1182,14 @@ class TrackingHandler(StorageHandler):
                 "Cannot download file at path %s, scheme %s not recognized"
                 % (str(manifest_entry.ref), str(url.scheme))
             )
-        return manifest_entry.path
+        # TODO(spencerpearson): should this go through util.to_native_slash_path
+        # instead of just getting typecast?
+        return util.FilePathStr(manifest_entry.path)
 
     def store_path(
         self,
         artifact: Artifact,
-        path: str,
+        path: Union[util.URIStr, util.FilePathStr],
         name: Optional[str] = None,
         checksum: bool = True,
         max_objects: Optional[int] = None,
@@ -1197,7 +1231,7 @@ class LocalFileHandler(StorageHandler):
         artifact: ArtifactInterface,
         manifest_entry: ArtifactEntry,
         local: bool = False,
-    ) -> str:
+    ) -> Union[util.URIStr, util.FilePathStr]:
         url = urlparse(manifest_entry.ref)
         local_path = f"{str(url.netloc)}{str(url.path)}"
         if not os.path.exists(local_path):
@@ -1206,7 +1240,7 @@ class LocalFileHandler(StorageHandler):
             )
 
         path, hit, cache_open = self._cache.check_md5_obj_path(
-            manifest_entry.digest,
+            util.B64MD5(manifest_entry.digest),  # TODO(spencerpearson): unsafe cast
             manifest_entry.size if manifest_entry.size is not None else 0,
         )
         if hit:
@@ -1228,7 +1262,7 @@ class LocalFileHandler(StorageHandler):
     def store_path(
         self,
         artifact: Artifact,
-        path: str,
+        path: Union[util.URIStr, util.FilePathStr],
         name: Optional[str] = None,
         checksum: bool = True,
         max_objects: Optional[int] = None,
@@ -1240,7 +1274,7 @@ class LocalFileHandler(StorageHandler):
         # Note, we follow symlinks for files contained within the directory
         entries = []
 
-        def md5(path: str) -> str:
+        def md5(path: str) -> util.B64MD5:
             return (
                 md5_file_b64(path)
                 if checksum
@@ -1265,13 +1299,16 @@ class LocalFileHandler(StorageHandler):
                             % max_objects
                         )
                     physical_path = os.path.join(root, sub_path)
+                    # TODO(spencerpearson): this is not a "logical path" in the sense that
+                    # `util.to_forward_slash_path` returns a "logical path"; it's a relative path
+                    # **on the local filesystem**.
                     logical_path = os.path.relpath(physical_path, start=local_path)
                     if name is not None:
                         logical_path = os.path.join(name, logical_path)
 
                     entry = ArtifactManifestEntry(
                         logical_path,
-                        os.path.join(path, logical_path),
+                        util.FilePathStr(os.path.join(path, logical_path)),
                         size=os.path.getsize(physical_path),
                         digest=md5(physical_path),
                     )
@@ -1311,11 +1348,11 @@ class S3Handler(StorageHandler):
     def init_boto(self) -> "boto3.resources.base.ServiceResource":
         if self._s3 is not None:
             return self._s3
-        boto3 = util.get_module(
+        boto: "boto3" = util.get_module(
             "boto3",
             required="s3:// references requires the boto3 library, run pip install wandb[aws]",
         )
-        self._s3 = boto3.session.Session().resource(
+        self._s3 = boto.session.Session().resource(
             "s3",
             endpoint_url=os.getenv("AWS_S3_ENDPOINT_URL"),
             region_name=os.getenv("AWS_REGION"),
@@ -1347,13 +1384,16 @@ class S3Handler(StorageHandler):
         artifact: ArtifactInterface,
         manifest_entry: ArtifactEntry,
         local: bool = False,
-    ) -> str:
+    ) -> Union[util.URIStr, util.FilePathStr]:
         if not local:
             assert manifest_entry.ref is not None
             return manifest_entry.ref
 
+        assert manifest_entry.ref is not None
+
         path, hit, cache_open = self._cache.check_etag_obj_path(
-            manifest_entry.digest,
+            util.URIStr(manifest_entry.ref),
+            util.ETag(manifest_entry.digest),  # TODO(spencerpearson): unsafe cast
             manifest_entry.size if manifest_entry.size is not None else 0,
         )
         if hit:
@@ -1361,7 +1401,6 @@ class S3Handler(StorageHandler):
 
         self.init_boto()
         assert self._s3 is not None  # mypy: unwraps optionality
-        assert manifest_entry.ref is not None
         bucket, key, _ = self._parse_uri(manifest_entry.ref)
         version = manifest_entry.extra.get("versionID")
 
@@ -1407,7 +1446,7 @@ class S3Handler(StorageHandler):
     def store_path(
         self,
         artifact: Artifact,
-        path: str,
+        path: Union[util.URIStr, util.FilePathStr],
         name: Optional[str] = None,
         checksum: bool = True,
         max_objects: Optional[int] = None,
@@ -1420,7 +1459,7 @@ class S3Handler(StorageHandler):
         # parsing. Once we have that, we can store the rest of the
         # metadata in the artifact entry itself.
         bucket, key, version = self._parse_uri(path)
-        path = f"{self.scheme}://{bucket}/{key}"
+        path = util.URIStr(f"{self.scheme}://{bucket}/{key}")
         if not self.versioning_enabled(bucket) and version:
             raise ValueError(
                 f"Specifying a versionId is not valid for s3://{bucket} as it does not have versioning enabled."
@@ -1528,15 +1567,15 @@ class S3Handler(StorageHandler):
             posix_ref = posix_path / relpath
         return ArtifactManifestEntry(
             str(posix_name),
-            f"{self.scheme}://{str(posix_ref)}",
-            self._etag_from_obj(obj),
+            util.URIStr(f"{self.scheme}://{str(posix_ref)}"),
+            util.ETag(self._etag_from_obj(obj)),
             size=self._size_from_obj(obj),
             extra=self._extra_from_obj(obj),
         )
 
     @staticmethod
-    def _etag_from_obj(obj: "boto3.s3.Object") -> str:
-        etag: str
+    def _etag_from_obj(obj: "boto3.s3.Object") -> util.ETag:
+        etag: util.ETag
         etag = obj.e_tag[1:-1]  # escape leading and trailing quote
         return etag
 
@@ -1551,11 +1590,13 @@ class S3Handler(StorageHandler):
         return extra
 
     @staticmethod
-    def _content_addressed_path(md5: str) -> str:
+    def _content_addressed_path(md5: str) -> util.FilePathStr:
         # TODO: is this the structure we want? not at all human
         # readable, but that's probably OK. don't want people
         # poking around in the bucket
-        return "wandb/%s" % base64.b64encode(md5.encode("ascii")).decode("ascii")
+        return util.FilePathStr(
+            "wandb/%s" % base64.b64encode(md5.encode("ascii")).decode("ascii")
+        )
 
 
 class GCSHandler(StorageHandler):
@@ -1604,13 +1645,13 @@ class GCSHandler(StorageHandler):
         artifact: ArtifactInterface,
         manifest_entry: ArtifactEntry,
         local: bool = False,
-    ) -> str:
+    ) -> Union[util.URIStr, util.FilePathStr]:
         if not local:
             assert manifest_entry.ref is not None
             return manifest_entry.ref
 
         path, hit, cache_open = self._cache.check_md5_obj_path(
-            manifest_entry.digest,
+            util.B64MD5(manifest_entry.digest),  # TODO(spencerpearson): unsafe cast
             manifest_entry.size if manifest_entry.size is not None else 0,
         )
         if hit:
@@ -1650,7 +1691,7 @@ class GCSHandler(StorageHandler):
     def store_path(
         self,
         artifact: Artifact,
-        path: str,
+        path: Union[util.URIStr, util.FilePathStr],
         name: Optional[str] = None,
         checksum: bool = True,
         max_objects: Optional[int] = None,
@@ -1662,7 +1703,7 @@ class GCSHandler(StorageHandler):
         # such as version identifiers, pare down the path to just the bucket
         # and key.
         bucket, key, version = self._parse_uri(path)
-        path = f"{self.scheme}://{bucket}/{key}"
+        path = util.URIStr(f"{self.scheme}://{bucket}/{key}")
         max_objects = max_objects or DEFAULT_MAX_OBJECTS
         if not self.versioning_enabled(bucket) and version:
             raise ValueError(
@@ -1743,7 +1784,7 @@ class GCSHandler(StorageHandler):
             posix_ref = posix_path / relpath
         return ArtifactManifestEntry(
             str(posix_name),
-            f"{self.scheme}://{str(posix_ref)}",
+            util.URIStr(f"{self.scheme}://{str(posix_ref)}"),
             obj.md5_hash,
             size=obj.size,
             extra=self._extra_from_obj(obj),
@@ -1757,11 +1798,13 @@ class GCSHandler(StorageHandler):
         }
 
     @staticmethod
-    def _content_addressed_path(md5: str) -> str:
+    def _content_addressed_path(md5: str) -> util.FilePathStr:
         # TODO: is this the structure we want? not at all human
         # readable, but that's probably OK. don't want people
         # poking around in the bucket
-        return "wandb/%s" % base64.b64encode(md5.encode("ascii")).decode("ascii")
+        return util.FilePathStr(
+            "wandb/%s" % base64.b64encode(md5.encode("ascii")).decode("ascii")
+        )
 
 
 class HTTPHandler(StorageHandler):
@@ -1779,24 +1822,27 @@ class HTTPHandler(StorageHandler):
         artifact: ArtifactInterface,
         manifest_entry: ArtifactEntry,
         local: bool = False,
-    ) -> str:
+    ) -> Union[util.URIStr, util.FilePathStr]:
         if not local:
             assert manifest_entry.ref is not None
             return manifest_entry.ref
 
+        assert manifest_entry.ref is not None
+
         path, hit, cache_open = self._cache.check_etag_obj_path(
-            manifest_entry.digest,
+            util.URIStr(manifest_entry.ref),
+            util.ETag(manifest_entry.digest),  # TODO(spencerpearson): unsafe cast
             manifest_entry.size if manifest_entry.size is not None else 0,
         )
         if hit:
             return path
 
-        assert manifest_entry.ref is not None
         response = self._session.get(manifest_entry.ref, stream=True)
         response.raise_for_status()
 
+        digest: Optional[Union[util.ETag, util.FilePathStr, util.URIStr]]
         digest, size, extra = self._entry_from_headers(response.headers)
-        digest = digest or path
+        digest = digest or manifest_entry.ref
         if manifest_entry.digest != digest:
             raise ValueError(
                 "Digest mismatch for url %s: expected %s but found %s"
@@ -1811,7 +1857,7 @@ class HTTPHandler(StorageHandler):
     def store_path(
         self,
         artifact: Artifact,
-        path: str,
+        path: Union[util.URIStr, util.FilePathStr],
         name: Optional[str] = None,
         checksum: bool = True,
         max_objects: Optional[int] = None,
@@ -1822,6 +1868,7 @@ class HTTPHandler(StorageHandler):
 
         with self._session.get(path, stream=True) as response:
             response.raise_for_status()
+            digest: Optional[Union[util.ETag, util.FilePathStr, util.URIStr]]
             digest, size, extra = self._entry_from_headers(response.headers)
             digest = digest or path
         return [
@@ -1830,7 +1877,7 @@ class HTTPHandler(StorageHandler):
 
     def _entry_from_headers(
         self, headers: requests.structures.CaseInsensitiveDict
-    ) -> Tuple[Optional[str], Optional[int], Dict[str, str]]:
+    ) -> Tuple[Optional[util.ETag], Optional[int], Dict[str, str]]:
         response_headers = {k.lower(): v for k, v in headers.items()}
         size = None
         if response_headers.get("content-length", None):
@@ -1875,7 +1922,7 @@ class WBArtifactHandler(StorageHandler):
         artifact: ArtifactInterface,
         manifest_entry: ArtifactEntry,
         local: bool = False,
-    ) -> str:
+    ) -> Union[util.URIStr, util.FilePathStr]:
         """
         Loads the file within the specified artifact given its
         corresponding entry. In this case, the referenced artifact is downloaded
@@ -1900,7 +1947,7 @@ class WBArtifactHandler(StorageHandler):
         dep_artifact = PublicArtifact.from_id(
             util.hex_to_b64_id(artifact_id), self.client
         )
-        link_target_path: str
+        link_target_path: util.FilePathStr
         if local:
             link_target_path = dep_artifact.get_path(artifact_file_path).download()
         else:
@@ -1911,7 +1958,7 @@ class WBArtifactHandler(StorageHandler):
     def store_path(
         self,
         artifact: Artifact,
-        path: str,
+        path: Union[util.URIStr, util.FilePathStr],
         name: Optional[str] = None,
         checksum: bool = True,
         max_objects: Optional[int] = None,
@@ -1947,8 +1994,10 @@ class WBArtifactHandler(StorageHandler):
             path = entry.ref
 
         # Create the path reference
-        path = "{}://{}/{}".format(
-            self._scheme, util.b64_to_hex_id(target_artifact.id), artifact_file_path
+        path = util.URIStr(
+            "{}://{}/{}".format(
+                self._scheme, util.b64_to_hex_id(target_artifact.id), artifact_file_path
+            )
         )
 
         # Return the new entry
@@ -1985,7 +2034,7 @@ class WBLocalArtifactHandler(StorageHandler):
         artifact: ArtifactInterface,
         manifest_entry: ArtifactEntry,
         local: bool = False,
-    ) -> str:
+    ) -> Union[util.URIStr, util.FilePathStr]:
         raise NotImplementedError(
             "Should not be loading a path for an artifact entry with unresolved client id."
         )
@@ -1993,7 +2042,7 @@ class WBLocalArtifactHandler(StorageHandler):
     def store_path(
         self,
         artifact: Artifact,
-        path: str,
+        path: Union[util.URIStr, util.FilePathStr],
         name: Optional[str] = None,
         checksum: bool = True,
         max_objects: Optional[int] = None,
@@ -2027,3 +2076,11 @@ class WBLocalArtifactHandler(StorageHandler):
                 digest=target_entry.digest,
             )
         ]
+
+
+class _ArtifactVersionType(Type):
+    name = "artifactVersion"
+    types = [Artifact, PublicArtifact]
+
+
+TypeRegistry.add(_ArtifactVersionType)
