@@ -13,6 +13,7 @@ import os
 from typing import Sequence, Union
 
 import wandb
+from wandb.env import _DISABLE_SERVICE, REQUIRE_SERVICE
 from wandb.errors import RequireError
 from wandb.sdk import wandb_run
 from wandb.sdk.lib.wburls import wburls
@@ -32,12 +33,18 @@ class _Requires:
         pass
 
     def _require_service(self) -> None:
-        os.environ["WANDB_REQUIRE_SERVICE"] = "True"
+        os.environ[REQUIRE_SERVICE] = "True"
         wandb.teardown = wandb._teardown  # type: ignore
         wandb.attach = wandb._attach  # type: ignore
         wandb_run.Run.detach = wandb_run.Run._detach  # type: ignore
 
     def require_service(self) -> None:
+        disable_service = os.environ.get(_DISABLE_SERVICE)
+        if disable_service:
+            if REQUIRE_SERVICE in os.environ:
+                del os.environ[REQUIRE_SERVICE]
+            return
+
         self._require_service()
 
     def _require_report_editing(self) -> None:
@@ -93,6 +100,6 @@ def _import_module_hook() -> None:
     """On wandb import, setup anything needed based on parent process require calls."""
     # TODO: optimize by caching which pids this has been done for or use real import hooks
     # TODO: make this more generic, but for now this works
-    req_service = os.environ.get("WANDB_REQUIRE_SERVICE")
+    req_service = os.environ.get(REQUIRE_SERVICE)
     if req_service:
         require("service")
