@@ -1,8 +1,5 @@
 import configparser
-from datetime import datetime
-from distutils.util import strtobool
 import enum
-from functools import reduce
 import getpass
 import json
 import multiprocessing
@@ -13,6 +10,9 @@ import socket
 import sys
 import tempfile
 import time
+from datetime import datetime
+from distutils.util import strtobool
+from functools import reduce
 from typing import (
     Any,
     Callable,
@@ -21,19 +21,19 @@ from typing import (
     ItemsView,
     Iterable,
     Mapping,
-    no_type_check,
     Optional,
     Sequence,
     Set,
     Tuple,
     Union,
+    no_type_check,
 )
 from urllib.parse import quote, urlencode, urlparse, urlsplit
 
 import wandb
+import wandb.env
 from wandb import util
 from wandb.apis.internal import Api
-import wandb.env
 from wandb.errors import UsageError
 from wandb.sdk.wandb_config import Config
 from wandb.sdk.wandb_setup import _EarlyLogger
@@ -122,7 +122,7 @@ def _get_program() -> Optional[Any]:
     if program is not None:
         return program
     try:
-        import __main__  # type: ignore
+        import __main__
 
         if __main__.__spec__ is None:
             return __main__.__file__
@@ -394,6 +394,7 @@ class Settings:
     _stats_pid: int  # (internal) base pid for system stats
     _stats_sample_rate_seconds: float
     _stats_samples_to_average: int
+    _stats_join_assets: bool  # join metrics from different assets before sending to backend
     _tmp_code_dir: str
     _tracelog: str
     _unsaved_keys: Sequence[str]
@@ -470,6 +471,7 @@ class Settings:
     start_method: str
     strict: bool
     summary_errors: int
+    summary_timeout: int
     summary_warnings: int
     sweep_id: str
     sweep_param_path: str
@@ -519,8 +521,9 @@ class Settings:
             },
             _platform={"value": util.get_platform_name()},
             _save_requirements={"value": True, "preprocessor": _str_as_bool},
-            _stats_sample_rate_seconds={"value": 2.0},
+            _stats_sample_rate_seconds={"value": 2.0, "preprocessor": float},
             _stats_samples_to_average={"value": 15},
+            _stats_join_assets={"value": True, "preprocessor": _str_as_bool},
             _tmp_code_dir={
                 "value": "code",
                 "hook": lambda x: self._path_convert(self.tmp_dir, x),
@@ -559,7 +562,7 @@ class Settings:
                 "value": tuple(),
                 "preprocessor": lambda x: tuple(x) if not isinstance(x, tuple) else x,
             },
-            init_timeout={"value": 30, "preprocessor": lambda x: int(x)},
+            init_timeout={"value": 60, "preprocessor": lambda x: int(x)},
             is_local={
                 "hook": (
                     lambda _: self.base_url != "https://api.wandb.ai"
@@ -605,6 +608,10 @@ class Settings:
                 "hook": lambda x: self._path_convert(self.wandb_dir, x),
             },
             resumed={"value": "False", "preprocessor": _str_as_bool},
+            root_dir={
+                "preprocessor": lambda x: str(x),
+                "value": os.path.abspath(os.getcwd()),
+            },
             run_mode={
                 "hook": lambda _: "offline-run" if self._offline else "run",
                 "auto_hook": True,
@@ -631,6 +638,7 @@ class Settings:
             silent={"value": "False", "preprocessor": _str_as_bool},
             start_method={"validator": self._validate_start_method},
             strict={"preprocessor": _str_as_bool},
+            summary_timeout={"value": 60, "preprocessor": lambda x: int(x)},
             summary_warnings={
                 "value": 5,
                 "preprocessor": lambda x: int(x),
@@ -1073,10 +1081,6 @@ class Settings:
         object.__setattr__(self, "_Settings_start_datetime", None)
         object.__setattr__(self, "_Settings_start_time", None)
 
-        if os.environ.get(wandb.env.DIR) is None:
-            # todo: double-check source, shouldn't it be Source.ENV?
-            self.update({"root_dir": os.path.abspath(os.getcwd())}, source=Source.BASE)
-
         # done with init, use self.update() to update attributes from now on
         self.__initialized = True
 
