@@ -15,21 +15,13 @@ import logging
 import os
 import sys
 import threading
-from typing import (
-    Any,
-    Dict,
-    List,
-    Optional,
-    Union,
-)
+from typing import Any, Dict, List, Optional, Union
 
 import wandb
 
-from . import wandb_manager
-from . import wandb_settings
+from . import wandb_manager, wandb_settings
 from .lib import config_util, server, tracelog
 
-
 # logger will be configured to be either a standard logger instance or _EarlyLogger
 logger = None
 
