@@ -15,10 +15,7 @@ from urllib.parse import quote as url_quote
 import wandb
 from wandb.proto import wandb_internal_pb2  # type: ignore
 from wandb.sdk.interface.interface_queue import InterfaceQueue
-from wandb.sdk.internal import datastore
-from wandb.sdk.internal import handler
-from wandb.sdk.internal import sender
-from wandb.sdk.internal import tb_watcher
+from wandb.sdk.internal import datastore, handler, sender, tb_watcher
 from wandb.util import check_and_warn_old, mkdir_exists_ok
 
 WANDB_SUFFIX = ".wandb"
@@ -129,7 +126,7 @@ class SyncThread(threading.Thread):
                 print(f"Found {tb_event_files} tfevent files in {tb_root}")
                 if len(tb_logdirs) > 3:
                     wandb.termwarn(
-                        "Found {} directories containing tfevent files. "
+                        f"Found {len(tb_logdirs)} directories containing tfevent files. "
                         "If these represent multiple experiments, sync them "
                         "individually or pass a list of paths."
                     )
